part of datepicker;

//// Calculate the visible dates count based on picker view
int _getViewDatesCount(DateRangePickerView pickerView, int numberOfWeeks) {
  if (pickerView == null) {
    return 0;
  }

  if (pickerView == DateRangePickerView.month) {
    return 7 * numberOfWeeks;
  }

  return 0;
}

bool _isRangeEquals(PickerDateRange range1, PickerDateRange range2) {
  if ((range1 == null && range2 != null) ||
      (range2 == null && range1 != null)) {
    return false;
  }

  if (range1 == range2 ||
      (isSameDate(range1.startDate, range2.startDate) &&
          isSameDate(range1.endDate, range2.endDate))) {
    return true;
  }

  return false;
}

bool _isDateRangesEquals(List<PickerDateRange> rangeCollection1,
    List<PickerDateRange> rangeCollection2) {
  if (rangeCollection1 == rangeCollection2) {
    return true;
  }

  if ((rangeCollection1 == null && rangeCollection2 != null) ||
      (rangeCollection2 == null && rangeCollection1 != null) ||
      (rangeCollection1.length != rangeCollection2.length)) {
    return false;
  }

  for (int i = 0; i < rangeCollection1.length; i++) {
    if (!_isRangeEquals(rangeCollection1[i], rangeCollection2[i])) {
      return false;
    }
  }

  return true;
}

//// Calculate the next view visible start date based on picker view.
DateTime _getNextViewStartDate(DateRangePickerView _pickerView,
    int numberOfWeeksInView, DateTime date, bool isRtl) {
  if (_pickerView == null) {
    return date;
  }

  if (isRtl != null && isRtl) {
    return _getPreviousViewStartDate(
        _pickerView, numberOfWeeksInView, date, false);
  }

  if (_pickerView == DateRangePickerView.month) {
    return numberOfWeeksInView == 6
        ? getNextMonthDate(date)
        : date.add(Duration(days: numberOfWeeksInView * 7));
  } else if (_pickerView == DateRangePickerView.year) {
    return _getNextYearDate(date, 1);
  } else if (_pickerView == DateRangePickerView.decade) {
    return _getNextYearDate(date, 10);
  } else if (_pickerView == DateRangePickerView.century) {
    return _getNextYearDate(date, 100);
  }

  return date;
}

//// Calculate the previous view visible start date based on calendar view.
DateTime _getPreviousViewStartDate(DateRangePickerView _pickerView,
    int numberOfWeeksInView, DateTime date, bool isRtl) {
  if (_pickerView == null) {
    return date;
  }

  if (isRtl != null && isRtl) {
    return _getNextViewStartDate(_pickerView, numberOfWeeksInView, date, false);
  }

  if (_pickerView == DateRangePickerView.month) {
    return numberOfWeeksInView == 6
        ? getPreviousMonthDate(date)
        : date.add(Duration(days: -numberOfWeeksInView * 7));
  } else if (_pickerView == DateRangePickerView.year) {
    return _getPreviousYearDate(date, 1);
  } else if (_pickerView == DateRangePickerView.decade) {
    return _getPreviousYearDate(date, 10);
  } else if (_pickerView == DateRangePickerView.century) {
    return _getPreviousYearDate(date, 100);
  }

  return date;
}

DateTime _getNextYearDate(DateTime date, int offset) {
  return DateTime(((date.year ~/ offset) * offset) + offset, 1, 1);
}

DateTime _getPreviousYearDate(DateTime date, int offset) {
  return DateTime(((date.year ~/ offset) * offset) - offset, 1, 1);
}

DateTime _getMonthStartDate(DateTime date) {
  return DateTime(date.year, date.month, 1);
}

DateTime _getMonthEndDate(DateTime date) {
  return getNextMonthDate(date).subtract(const Duration(days: 1));
}

int _isDateIndexInCollection(List<DateTime> dates, DateTime date) {
  if (dates == null || date == null) {
    return -1;
  }

  for (int i = 0; i < dates.length; i++) {
    final DateTime visibleDate = dates[i];
    if (isSameDate(visibleDate, date)) {
      return i;
    }
  }

  return -1;
}

bool _isDateCollectionEquals(
    List<DateTime> datesCollection1, List<DateTime> datesCollection2) {
  if (datesCollection1 == datesCollection2) {
    return true;
  }

  if ((datesCollection1 == null && datesCollection2 != null) ||
      (datesCollection2 == null && datesCollection1 != null) ||
      (datesCollection1.length != datesCollection2.length)) {
    return false;
  }

  for (int i = 0; i < datesCollection1.length; i++) {
    if (!isSameDate(datesCollection1[i], datesCollection2[i])) {
      return false;
    }
  }

  return true;
}

bool _isEnabledDate(
    DateTime startDate, DateTime endDate, bool enablePastDates, DateTime date) {
  return isDateWithInDateRange(startDate, endDate, date) &&
      (enablePastDates ||
          (!enablePastDates &&
              isDateWithInDateRange(DateTime.now(), endDate, date)));
}

bool _isDateAsCurrentMonthDate(DateTime visibleDate, int rowCount,
    bool showLeadingAndTrialingDates, DateTime date) {
  if (rowCount == 6 &&
      !showLeadingAndTrialingDates &&
      date.month != visibleDate.month) {
    return false;
  }

  return true;
}

Map<String, double> _getTopAndLeftValues(bool _isRtl, double _left, double _top,
    double _cellWidth, double _cellHeight, double _width) {
  final Map<String, double> topAndLeft = <String, double>{
    'left': _left,
    'top': _top
  };
  if (_isRtl) {
    if (_left.round() == _cellWidth.round()) {
      _left = 0;
    } else {
      _left -= _cellWidth;
    }
    if (_left < 0) {
      _left = _width - _cellWidth;
      _top += _cellHeight;
    }
  } else {
    _left += _cellWidth;
    if (_left.round() == _width.round()) {
      _top += _cellHeight;
      _left = 0;
    }
  }
  topAndLeft['left'] = _left;
  topAndLeft['top'] = _top;

  return topAndLeft;
}

bool _isDateWithInVisibleDates(
    List<DateTime> visibleDates, List<DateTime> dates, DateTime date) {
  if (dates == null || dates.isEmpty) {
    return false;
  }

  final DateTime visibleStartDate = visibleDates[0];
  final DateTime visibleEndDate = visibleDates[visibleDates.length - 1];
  for (DateTime currentDate in dates) {
    if (!isDateWithInDateRange(visibleStartDate, visibleEndDate, currentDate)) {
      continue;
    }

    if (isSameDate(currentDate, date)) {
      return true;
    }
  }

  return false;
}

bool _isWeekend(List<int> weekendIndex, DateTime date) {
  if (weekendIndex == null || weekendIndex.isEmpty) {
    return false;
  }

  return weekendIndex.contains(date.weekday);
}

bool _canMoveToPreviousViewRtl(
    DateRangePickerView _view,
    int _numberOfWeeksInView,
    DateTime _minDate,
    DateTime _maxDate,
    List<DateTime> _visibleDates,
    bool isRtl) {
  if (isRtl) {
    return _canMoveToNextView(
        _view, _numberOfWeeksInView, _maxDate, _visibleDates);
  } else {
    return _canMoveToPreviousView(
        _view, _numberOfWeeksInView, _minDate, _visibleDates);
  }
}

bool _canMoveToNextViewRtl(
    DateRangePickerView _view,
    int _numberOfWeeksInView,
    DateTime _minDate,
    DateTime _maxDate,
    List<DateTime> _visibleDates,
    bool isRtl) {
  if (isRtl) {
    return _canMoveToPreviousView(
        _view, _numberOfWeeksInView, _minDate, _visibleDates);
  } else {
    return _canMoveToNextView(
        _view, _numberOfWeeksInView, _maxDate, _visibleDates);
  }
}

bool _canMoveToPreviousView(DateRangePickerView _view, int _numberOfWeeksInView,
    DateTime _minDate, List<DateTime> _visibleDates,
    [List<int> nonWorkingDays]) {
  if (_view == DateRangePickerView.month && _numberOfWeeksInView != 6) {
    DateTime _prevViewDate = _visibleDates[0];
    _prevViewDate = _prevViewDate.subtract(const Duration(days: 1));
    if (!isSameOrAfterDate(_minDate, _prevViewDate)) {
      return false;
    }
  } else if (_view == DateRangePickerView.month) {
    final DateTime _currentDate = _visibleDates[_visibleDates.length ~/ 2];
    final DateTime _previousDate = getPreviousMonthDate(_currentDate);
    if ((_previousDate.month < _minDate.month &&
            _previousDate.year == _minDate.year) ||
        _previousDate.year < _minDate.year) {
      return false;
    }
  } else {
    final int _currentYear = _visibleDates[_visibleDates.length ~/ 2].year;
    int offset = 1;
    if (_view == DateRangePickerView.decade) {
      offset = 10;
    } else if (_view == DateRangePickerView.century) {
      offset = 100;
    }

    if (((_currentYear ~/ offset) * offset) - offset <
        ((_minDate.year ~/ offset) * offset)) {
      return false;
    }
  }
  return true;
}

//// Get the visible dates based on the date value and visible dates count.
List<DateTime> _getVisibleYearDates(DateTime date, DateRangePickerView view) {
  final List<DateTime> datesCollection = <DateTime>[];
  DateTime currentDate;
  const int daysCount = 12;
  if (view == DateRangePickerView.year) {
    for (int i = 1; i <= daysCount; i++) {
      currentDate = DateTime(date.year, i, 1);
      datesCollection.add(currentDate);
    }
  } else if (view == DateRangePickerView.decade) {
    final int year = (date.year ~/ 10) * 10;
    for (int i = 0; i < daysCount; i++) {
      currentDate = DateTime(year + i, 1, 1);
      datesCollection.add(currentDate);
    }
  } else if (view == DateRangePickerView.century) {
    final int year = (date.year ~/ 100) * 100;
    for (int i = 0; i < daysCount; i++) {
      currentDate = DateTime(year + (i * 10), 1, 1);
      datesCollection.add(currentDate);
    }
  }

  return datesCollection;
}

bool _canMoveToNextView(DateRangePickerView _view, int _numberOfWeeksInView,
    DateTime _maxDate, List<DateTime> _visibleDates,
    [List<int> nonWorkingDays]) {
  if (_view == DateRangePickerView.month && _numberOfWeeksInView != 6) {
    DateTime _nextViewDate = _visibleDates[_visibleDates.length - 1];
    _nextViewDate = _nextViewDate.add(const Duration(days: 1));
    if (!isSameOrBeforeDate(_maxDate, _nextViewDate)) {
      return false;
    }
  } else if (_view == DateRangePickerView.month) {
    final DateTime _currentDate = _visibleDates[_visibleDates.length ~/ 2];
    final DateTime _nextDate = getNextMonthDate(_currentDate);
    if ((_nextDate.month > _maxDate.month && _nextDate.year == _maxDate.year) ||
        _nextDate.year > _maxDate.year) {
      return false;
    }
  } else {
    final int _currentYear = _visibleDates[_visibleDates.length ~/ 2].year;
    int offset = 1;
    if (_view == DateRangePickerView.decade) {
      offset = 10;
    } else if (_view == DateRangePickerView.century) {
      offset = 100;
    }

    if (((_currentYear ~/ offset) * offset) + offset >
        ((_maxDate.year ~/ offset) * offset)) {
      return false;
    }
  }
  return true;
}
