part of datepicker;

class _PickerHeaderPainter extends CustomPainter {
  _PickerHeaderPainter(
      this.visibleDates,
      this.headerStyle,
      this.view,
      this.numberOfWeeksInView,
      this.monthFormat,
      this._datePickerTheme,
      this.isRtl,
      this._locale,
      this._hovering)
      : super(repaint: visibleDates);

  final DateRangePickerHeaderStyle headerStyle;
  final DateRangePickerView view;
  final int numberOfWeeksInView;
  final SfDateRangePickerThemeData _datePickerTheme;
  final bool isRtl;
  final String monthFormat;
  final bool _hovering;
  ValueNotifier<List<DateTime>> visibleDates;
  String _headerText;
  TextPainter _textPainter;
  final Locale _locale;

  @override
  void paint(Canvas canvas, Size size) {
    canvas.clipRect(Rect.fromLTWH(0, 0, size.width, size.height));
    double _xPosition = 10.0;
    _textPainter = _textPainter ?? TextPainter();
    _textPainter.textDirection = TextDirection.ltr;
    _textPainter.textWidthBasis = TextWidthBasis.longestLine;
    _textPainter.maxLines = 1;

    _headerText = _getHeaderText();

    TextStyle style = headerStyle.textStyle ?? _datePickerTheme.headerTextStyle;
    if (_hovering) {
      style = style.copyWith(color: Colors.blue);
    }

    final TextSpan span = TextSpan(text: _headerText, style: style);
    _textPainter.text = span;

    if (headerStyle.textAlign == TextAlign.justify) {
      _textPainter.textAlign = headerStyle.textAlign;
    }

    _textPainter.layout(
        minWidth: 0,
        maxWidth: size.width - _xPosition > 0 ? size.width - _xPosition : 0);

    if (headerStyle.textAlign == TextAlign.center) {
      _xPosition = size.width / 2 - _textPainter.width / 2;
    } else if ((!isRtl &&
            (headerStyle.textAlign == TextAlign.right ||
                headerStyle.textAlign == TextAlign.end)) ||
        (isRtl &&
            (headerStyle.textAlign == TextAlign.left ||
                headerStyle.textAlign == TextAlign.start))) {
      _xPosition = size.width - _textPainter.width - _xPosition;
    }

    _textPainter.paint(
        canvas, Offset(_xPosition, size.height / 2 - _textPainter.height / 2));
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    final _PickerHeaderPainter oldWidget = oldDelegate;
    return oldWidget.headerStyle != headerStyle ||
        oldWidget.isRtl != isRtl ||
        oldWidget.numberOfWeeksInView != numberOfWeeksInView ||
        oldWidget._locale != _locale ||
        oldWidget._datePickerTheme != _datePickerTheme ||
        (kIsWeb && oldWidget._hovering != _hovering);
  }

  String _getHeaderText() {
    final List<DateTime> _dates = visibleDates.value;
    if (view == DateRangePickerView.month &&
        numberOfWeeksInView != 6 &&
        _dates[0].month != _dates[_dates.length - 1].month) {
      final String monthTextFormat =
          monthFormat == null || monthFormat.isEmpty ? 'MMM' : monthFormat;
      return DateFormat(monthTextFormat, _locale.toString())
              .format(_dates[0])
              .toString() +
          ' ' +
          _dates[0].year.toString() +
          ' - ' +
          DateFormat(monthTextFormat, _locale.toString())
              .format(_dates[_dates.length - 1])
              .toString() +
          ' ' +
          _dates[_dates.length - 1].year.toString();
    } else if (view == DateRangePickerView.month) {
      final String monthTextFormat =
          monthFormat == null || monthFormat.isEmpty ? 'MMMM' : monthFormat;
      return DateFormat(monthTextFormat, _locale.toString())
              .format(_dates[_dates.length ~/ 2])
              .toString() +
          ' ' +
          _dates[_dates.length ~/ 2].year.toString();
    } else if (view == DateRangePickerView.year) {
      final DateTime date = _dates[_dates.length ~/ 2];
      return date.year.toString();
    } else if (view == DateRangePickerView.decade) {
      final int year = (_dates[_dates.length ~/ 2].year ~/ 10) * 10;
      return year.toString() + ' - ' + (year + 9).toString();
    } else if (view == DateRangePickerView.century) {
      final int year = (_dates[_dates.length ~/ 2].year ~/ 100) * 100;
      return year.toString() + ' - ' + (year + 99).toString();
    }

    return '';
  }

  /// overrides this property to build the semantics information which uses to
  /// return the required information for accessibility, need to return the list
  /// of custom painter semantics which contains the rect area and the semantics
  /// properties for accessibility
  @override
  SemanticsBuilderCallback get semanticsBuilder {
    return (Size size) {
      final dynamic rect = Offset.zero & size;
      return <CustomPainterSemantics>[
        CustomPainterSemantics(
          rect: rect,
          properties: SemanticsProperties(
            label: _headerText != null ? _headerText.replaceAll('-', 'to') : '',
            textDirection: TextDirection.ltr,
          ),
        ),
      ];
    };
  }

  @override
  bool shouldRebuildSemantics(CustomPainter oldDelegate) {
    return true;
  }
}
