part of datepicker;

class _PickerHeaderView extends StatefulWidget {
  const _PickerHeaderView(
      this.visibleDates,
      this.headerStyle,
      this.selectionMode,
      this.view,
      this.numberOfWeeksInView,
      this.showNavigationArrow,
      this.navigationDirection,
      this.enableSwipeSelection,
      this.minDate,
      this.maxDate,
      this.monthFormat,
      this._datePickerTheme,
      this._locale,
      this.width,
      this.height,
      this.previousNavigationCallback,
      this.nextNavigationCallback,
      this.isRtl,
      {Key key})
      : super(key: key);

  final DateRangePickerSelectionMode selectionMode;
  final DateRangePickerHeaderStyle headerStyle;
  final DateRangePickerView view;
  final int numberOfWeeksInView;
  final bool showNavigationArrow;
  final DateRangePickerNavigationDirection navigationDirection;
  final DateTime minDate;
  final DateTime maxDate;
  final String monthFormat;
  final bool enableSwipeSelection;
  final SfDateRangePickerThemeData _datePickerTheme;
  final Locale _locale;
  final ValueNotifier<List<DateTime>> visibleDates;
  final VoidCallback previousNavigationCallback;
  final VoidCallback nextNavigationCallback;
  final double width;
  final double height;
  final bool isRtl;

  @override
  _PickerHeaderViewState createState() => _PickerHeaderViewState();
}

class _PickerHeaderViewState extends State<_PickerHeaderView> {
  bool _hovering;

  @override
  void initState() {
    _hovering = false;
    _addListener();
    super.initState();
  }

  void listener() {
    if (!mounted) {
      return;
    }

    if (widget.showNavigationArrow ||
        (widget.view == DateRangePickerView.month &&
            widget.enableSwipeSelection &&
            (widget.selectionMode == DateRangePickerSelectionMode.range ||
                widget.selectionMode ==
                    DateRangePickerSelectionMode.multiRange))) {
      setState(() {});
    }
  }

  void _addListener() {
    SchedulerBinding.instance.addPostFrameCallback((_) {
      widget.visibleDates.addListener(listener);
    });
  }

  @override
  void didUpdateWidget(_PickerHeaderView oldWidget) {
    widget.visibleDates.removeListener(listener);
    _addListener();
    super.didUpdateWidget(oldWidget);
  }

  @override
  void dispose() {
    widget.visibleDates.removeListener(listener);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    double _arrowWidth = 0;
    double _headerWidth = widget.width;
    if (widget.showNavigationArrow ||
        (widget.view == DateRangePickerView.month &&
            widget.enableSwipeSelection &&
            (widget.selectionMode == DateRangePickerSelectionMode.range ||
                widget.selectionMode ==
                    DateRangePickerSelectionMode.multiRange))) {
      _arrowWidth = widget.width / 6;
      _headerWidth = _arrowWidth * 4;
    }

    final Color arrowColor = widget.headerStyle.textStyle != null
        ? widget.headerStyle.textStyle.color
        : (widget._datePickerTheme.headerTextStyle.color);
    Color _prevArrowColor = arrowColor;
    Color _nextArrowColor = arrowColor;
    final List<DateTime> _dates = widget.visibleDates.value;
    if (!_canMoveToNextView(
        widget.view, widget.numberOfWeeksInView, widget.maxDate, _dates)) {
      _nextArrowColor = _nextArrowColor.withOpacity(0.35);
    }

    if (!_canMoveToPreviousView(
        widget.view, widget.numberOfWeeksInView, widget.minDate, _dates)) {
      _prevArrowColor = _prevArrowColor.withOpacity(0.35);
    }

    final Widget _headerText = MouseRegion(
        onEnter: (PointerEnterEvent event) {
          if (widget.view == DateRangePickerView.century) {
            return;
          }

          setState(() {
            _hovering = true;
          });
        },
        onHover: (PointerHoverEvent event) {
          if (widget.view == DateRangePickerView.century) {
            return;
          }

          setState(() {
            _hovering = true;
          });
        },
        onExit: (PointerExitEvent event) {
          setState(() {
            _hovering = false;
          });
        },
        child: RepaintBoundary(
            child: CustomPaint(
          // Returns the header view  as a child for the calendar.
          painter: _PickerHeaderPainter(
              widget.visibleDates,
              widget.headerStyle,
              widget.view,
              widget.numberOfWeeksInView,
              widget.monthFormat,
              widget._datePickerTheme,
              widget.isRtl,
              widget._locale,
              _hovering),
          size: Size(_headerWidth, widget.height),
        )));

    final Container _leftArrow = Container(
      alignment: Alignment.center,
      color: widget.headerStyle.backgroundColor ??
          widget._datePickerTheme.headerBackgroundColor,
      width: _arrowWidth,
      padding: const EdgeInsets.all(0),
      child: FlatButton(
        //// set splash color as transparent when arrow reaches min date(disabled)
        splashColor: _prevArrowColor != arrowColor ? Colors.transparent : null,
        color: widget.headerStyle.backgroundColor ??
            widget._datePickerTheme.headerBackgroundColor,
        onPressed: widget.previousNavigationCallback,
        padding: const EdgeInsets.all(0),
        child: Semantics(
          label: 'Backward',
          child: Icon(
            widget.navigationDirection ==
                    DateRangePickerNavigationDirection.horizontal
                ? Icons.chevron_left
                : Icons.keyboard_arrow_up,
            color: _prevArrowColor,
            size: widget.height * 0.5,
          ),
        ),
      ),
    );

    final Container _rightArrow = Container(
      alignment: Alignment.center,
      color: widget.headerStyle.backgroundColor ??
          widget._datePickerTheme.headerBackgroundColor,
      width: _arrowWidth,
      padding: const EdgeInsets.all(0),
      child: FlatButton(
        //// set splash color as transparent when arrow reaches max date(disabled)
        splashColor: _nextArrowColor != arrowColor ? Colors.transparent : null,
        color: widget.headerStyle.backgroundColor ??
            widget._datePickerTheme.headerBackgroundColor,
        onPressed: widget.nextNavigationCallback,
        padding: const EdgeInsets.all(0),
        child: Semantics(
          label: 'Forward',
          child: Icon(
            widget.navigationDirection ==
                    DateRangePickerNavigationDirection.horizontal
                ? Icons.chevron_right
                : Icons.keyboard_arrow_down,
            color: _nextArrowColor,
            size: widget.height * 0.5,
          ),
        ),
      ),
    );

    if (widget.headerStyle.textAlign == TextAlign.left ||
        widget.headerStyle.textAlign == TextAlign.start) {
      return Row(
          mainAxisAlignment: MainAxisAlignment.center,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: <Widget>[
            _headerText,
            _leftArrow,
            _rightArrow,
          ]);
    } else if (widget.headerStyle.textAlign == TextAlign.right ||
        widget.headerStyle.textAlign == TextAlign.end) {
      return Row(
          mainAxisAlignment: MainAxisAlignment.center,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: <Widget>[
            _leftArrow,
            _rightArrow,
            _headerText,
          ]);
    } else {
      return Row(
          mainAxisAlignment: MainAxisAlignment.center,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: <Widget>[
            _leftArrow,
            _headerText,
            _rightArrow,
          ]);
    }
  }
}
