part of datepicker;

class _PickerViewHeaderPainter extends CustomPainter {
  _PickerViewHeaderPainter(
      this._visibleDates,
      this._viewHeaderStyle,
      this._viewHeaderHeight,
      this._monthViewSettings,
      this._datePickerTheme,
      this._locale,
      this._isRtl,
      this._monthCellStyle,
      this._todayHighlightColor);

  final DateRangePickerViewHeaderStyle _viewHeaderStyle;
  final DateRangePickerMonthViewSettings _monthViewSettings;
  final List<DateTime> _visibleDates;
  final double _viewHeaderHeight;
  final DateRangePickerMonthCellStyle _monthCellStyle;
  final Locale _locale;
  final bool _isRtl;
  final Color _todayHighlightColor;
  TextPainter _textPainter;
  final SfDateRangePickerThemeData _datePickerTheme;

  @override
  void paint(Canvas canvas, Size size) {
    canvas.clipRect(Rect.fromLTWH(0, 0, size.width, size.height));
    final double _width = size.width / 7;

    /// Initializes the default text style for the texts in view header of calendar.
    final TextStyle _viewHeaderDayStyle =
        _viewHeaderStyle.textStyle ?? _datePickerTheme.viewHeaderTextStyle;
    final DateTime today = DateTime.now();
    TextStyle dayTextStyle = _viewHeaderDayStyle;
    double _xPosition = 0;
    double _yPosition = 0;
    DateTime _currentDate;
    const int _daysCount = 7;
    final bool _hasToday = _monthViewSettings.numberOfWeeksInView > 0 &&
            _monthViewSettings.numberOfWeeksInView < 6
        ? true
        : (_visibleDates[_visibleDates.length ~/ 2].month == today.month &&
                _visibleDates[_visibleDates.length ~/ 2].year == today.year)
            ? true
            : false;
    for (int i = 0; i < _daysCount; i++) {
      _currentDate = _visibleDates[_isRtl ? _getRtlIndex(_daysCount, i) : i];
      String _dayText =
          DateFormat(_monthViewSettings.dayFormat, _locale.toString())
              .format(_currentDate)
              .toString()
              .toUpperCase();
      _dayText = _updateViewHeaderFormat(_dayText);

      if (_hasToday &&
          _currentDate.weekday == today.weekday &&
          isDateWithInDateRange(_visibleDates[0],
              _visibleDates[_visibleDates.length - 1], today)) {
        final Color textColor = _monthCellStyle.todayTextStyle != null &&
                _monthCellStyle.todayTextStyle.color != null
            ? _monthCellStyle.todayTextStyle.color
            : _todayHighlightColor ?? _datePickerTheme.todayHighlightColor;
        dayTextStyle = _viewHeaderDayStyle.copyWith(color: textColor);
      } else {
        dayTextStyle = _viewHeaderDayStyle;
      }

      final TextSpan dayTextSpan = TextSpan(
        text: _dayText,
        style: dayTextStyle,
      );

      _textPainter = _textPainter ??
          TextPainter(
              textDirection: TextDirection.ltr,
              textAlign: TextAlign.left,
              textWidthBasis: TextWidthBasis.longestLine);
      _textPainter.text = dayTextSpan;
      _textPainter.layout(minWidth: 0, maxWidth: _width);
      _yPosition = (_viewHeaderHeight - _textPainter.height) / 2;
      _textPainter.paint(
          canvas,
          Offset(
              _xPosition + (_width / 2 - _textPainter.width / 2), _yPosition));
      _xPosition += _width;
    }
  }

  String _updateViewHeaderFormat(String _dayText) {
    //// EE format value shows the week days as S, M, T, W, T, F, S.
    if (_monthViewSettings.dayFormat == 'EE') {
      _dayText = _dayText[0];
    }

    return _dayText;
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    final _PickerViewHeaderPainter oldWidget = oldDelegate;
    return oldWidget._visibleDates != _visibleDates ||
        oldWidget._viewHeaderStyle != _viewHeaderStyle ||
        oldWidget._viewHeaderHeight != _viewHeaderHeight ||
        oldWidget._todayHighlightColor != _todayHighlightColor ||
        oldWidget._monthViewSettings != _monthViewSettings ||
        oldWidget._datePickerTheme != _datePickerTheme ||
        oldWidget._isRtl != _isRtl ||
        oldWidget._locale != _locale;
  }

  List<CustomPainterSemantics> _getSemanticsBuilder(Size size) {
    final List<CustomPainterSemantics> _semanticsBuilder =
        <CustomPainterSemantics>[];
    double _left, _top, _cellWidth;
    _cellWidth = size.width / 7;
    _left = _isRtl ? size.width - _cellWidth : 0;
    _top = 0;
    for (int i = 0; i < 7; i++) {
      _semanticsBuilder.add(CustomPainterSemantics(
        rect: Rect.fromLTWH(_left, _top, _cellWidth, size.height),
        properties: SemanticsProperties(
          label: DateFormat('EEEEE')
              .format(_visibleDates[i])
              .toString()
              .toUpperCase(),
          textDirection: TextDirection.ltr,
        ),
      ));
      if (_isRtl) {
        _left -= _cellWidth;
      } else {
        _left += _cellWidth;
      }
    }

    return _semanticsBuilder;
  }

  /// overrides this property to build the semantics information which uses to
  /// return the required information for accessibility, need to return the list
  /// of custom painter semantics which contains the rect area and the semantics
  /// properties for accessibility
  @override
  SemanticsBuilderCallback get semanticsBuilder {
    return (Size size) {
      return _getSemanticsBuilder(size);
    };
  }

  @override
  bool shouldRebuildSemantics(CustomPainter oldDelegate) {
    final _PickerViewHeaderPainter _oldWidget = oldDelegate;
    return _oldWidget._visibleDates != _visibleDates;
  }
}
