part of datepicker;

class _PickerScrollView extends StatefulWidget {
  const _PickerScrollView(this.picker, this.controller, this.width, this.height,
      this.isRtl, this._datePickerTheme, this._locale,
      {Key key, this.getPickerStateValues, this.updatePickerStateValues})
      : super(key: key);

  final SfDateRangePicker picker;
  final double width;
  final double height;
  final bool isRtl;
  final _UpdatePickerState getPickerStateValues;
  final _UpdatePickerState updatePickerStateValues;
  final DateRangePickerController controller;
  final SfDateRangePickerThemeData _datePickerTheme;
  final Locale _locale;

  @override
  _PickerScrollViewState createState() => _PickerScrollViewState();
}

class _PickerScrollViewState extends State<_PickerScrollView>
    with TickerProviderStateMixin {
  // three views to arrange the view in vertical/horizontal direction and handle the swiping
  _PickerView _currentView, _nextView, _previousView;

  // the three children which to be added into the layout
  List<_PickerView> _children;

  // holds the index of the current displaying view
  int _currentChildIndex;

  // _scrollStartPosition contains the touch movement starting position
  // _position contains distance that the view swiped
  double _scrollStartPosition, _position;

  // animation controller to control the animation
  AnimationController _animationController;

  // animation handled for the view swiping
  Animation<double> _animation;

  // tween animation to handle the animation
  Tween<double> _tween;

  // three visible dates for the three views, the dates will updated based on the swiping in the swipe end
  // _currentViewVisibleDates which stores the visible dates of the current displaying view
  List<DateTime> _visibleDates,
      _previousViewVisibleDates,
      _nextViewVisibleDates,
      _currentViewVisibleDates;

  /// keys maintained to access the data and methods from the calendar view class.
  dynamic _previousViewKey, _currentViewKey, _nextViewKey;

  _PickerStateArgs _pickerStateDetails;
  FocusNode _focusNode;

  @override
  void initState() {
    _previousViewKey = GlobalKey<_PickerViewState>();
    _currentViewKey = GlobalKey<_PickerViewState>();
    _nextViewKey = GlobalKey<_PickerViewState>();
    _focusNode = FocusNode();
    _pickerStateDetails = _PickerStateArgs();
    _currentChildIndex = 1;
    _updateVisibleDates();
    _animationController = AnimationController(
        duration: const Duration(milliseconds: 250),
        vsync: this,
        animationBehavior: AnimationBehavior.normal);
    _tween = Tween<double>(begin: 0.0, end: 0.1);
    _animation = _tween.animate(_animationController)
      ..addListener(animationListener);

    super.initState();
  }

  void _updateVisibleDates() {
    _pickerStateDetails._currentDate = null;
    widget.getPickerStateValues(_pickerStateDetails);
    final DateTime currentDate = _pickerStateDetails._currentDate;
    final DateTime prevDate = _getPreviousViewStartDate(
        widget.controller.view,
        widget.picker.monthViewSettings.numberOfWeeksInView,
        _pickerStateDetails._currentDate,
        widget.isRtl);
    final DateTime nextDate = _getNextViewStartDate(
        widget.controller.view,
        widget.picker.monthViewSettings.numberOfWeeksInView,
        _pickerStateDetails._currentDate,
        widget.isRtl);

    if (widget.controller.view == DateRangePickerView.month) {
      _visibleDates = getVisibleDates(
          currentDate,
          null,
          widget.picker.monthViewSettings.firstDayOfWeek,
          _getViewDatesCount(widget.controller.view,
              widget.picker.monthViewSettings.numberOfWeeksInView));
      _previousViewVisibleDates = getVisibleDates(
          prevDate,
          null,
          widget.picker.monthViewSettings.firstDayOfWeek,
          _getViewDatesCount(widget.controller.view,
              widget.picker.monthViewSettings.numberOfWeeksInView));
      _nextViewVisibleDates = getVisibleDates(
          nextDate,
          null,
          widget.picker.monthViewSettings.firstDayOfWeek,
          _getViewDatesCount(widget.controller.view,
              widget.picker.monthViewSettings.numberOfWeeksInView));
    } else {
      _visibleDates = _getVisibleYearDates(currentDate, widget.controller.view);
      _previousViewVisibleDates =
          _getVisibleYearDates(prevDate, widget.controller.view);
      _nextViewVisibleDates =
          _getVisibleYearDates(nextDate, widget.controller.view);
    }

    _currentViewVisibleDates = _visibleDates;
    _pickerStateDetails._currentViewVisibleDates = _currentViewVisibleDates;
    widget.updatePickerStateValues(_pickerStateDetails);

    if (_currentChildIndex == 0) {
      _visibleDates = _nextViewVisibleDates;
      _nextViewVisibleDates = _previousViewVisibleDates;
      _previousViewVisibleDates = _currentViewVisibleDates;
    } else if (_currentChildIndex == 1) {
      _visibleDates = _currentViewVisibleDates;
    } else if (_currentChildIndex == 2) {
      _visibleDates = _previousViewVisibleDates;
      _previousViewVisibleDates = _nextViewVisibleDates;
      _nextViewVisibleDates = _currentViewVisibleDates;
    }
  }

  void _updateNextViewVisibleDates() {
    DateTime _currentViewDate = _currentViewVisibleDates[0];
    if ((widget.controller.view == DateRangePickerView.month &&
            widget.picker.monthViewSettings.numberOfWeeksInView == 6) ||
        widget.controller.view == DateRangePickerView.year ||
        widget.controller.view == DateRangePickerView.decade ||
        widget.controller.view == DateRangePickerView.century) {
      _currentViewDate = _currentViewVisibleDates[
          (_currentViewVisibleDates.length / 2).truncate()];
    }

    _currentViewDate = _getNextViewStartDate(
        widget.controller.view,
        widget.picker.monthViewSettings.numberOfWeeksInView,
        _currentViewDate,
        widget.isRtl);
    List<DateTime> _dates;
    if (widget.controller.view == DateRangePickerView.month) {
      _dates = getVisibleDates(
          _currentViewDate,
          null,
          widget.picker.monthViewSettings.firstDayOfWeek,
          _getViewDatesCount(widget.controller.view,
              widget.picker.monthViewSettings.numberOfWeeksInView));
    } else {
      _dates = _getVisibleYearDates(_currentViewDate, widget.controller.view);
    }

    if (_currentChildIndex == 0) {
      _nextViewVisibleDates = _dates;
    } else if (_currentChildIndex == 1) {
      _previousViewVisibleDates = _dates;
    } else {
      _visibleDates = _dates;
    }
  }

  void _updatePreviousViewVisibleDates() {
    DateTime _currentViewDate = _currentViewVisibleDates[0];
    if ((widget.controller.view == DateRangePickerView.month &&
            widget.picker.monthViewSettings.numberOfWeeksInView == 6) ||
        widget.controller.view == DateRangePickerView.year ||
        widget.controller.view == DateRangePickerView.decade ||
        widget.controller.view == DateRangePickerView.century) {
      _currentViewDate = _currentViewVisibleDates[
          (_currentViewVisibleDates.length / 2).truncate()];
    }

    _currentViewDate = _getPreviousViewStartDate(
        widget.controller.view,
        widget.picker.monthViewSettings.numberOfWeeksInView,
        _currentViewDate,
        widget.isRtl);
    List<DateTime> _dates;
    if (widget.controller.view == DateRangePickerView.month) {
      _dates = getVisibleDates(
          _currentViewDate,
          null,
          widget.picker.monthViewSettings.firstDayOfWeek,
          _getViewDatesCount(widget.controller.view,
              widget.picker.monthViewSettings.numberOfWeeksInView));
    } else {
      _dates = _getVisibleYearDates(_currentViewDate, widget.controller.view);
    }

    if (_currentChildIndex == 0) {
      _visibleDates = _dates;
    } else if (_currentChildIndex == 1) {
      _nextViewVisibleDates = _dates;
    } else {
      _previousViewVisibleDates = _dates;
    }
  }

  void _getPickerViewStateDetails(_PickerStateArgs details) {
    details._currentViewVisibleDates = _currentViewVisibleDates;
    _pickerStateDetails._currentDate = null;
    widget.getPickerStateValues(_pickerStateDetails);
    details._currentDate = _pickerStateDetails._currentDate;
    details._selectedDate = _pickerStateDetails._selectedDate;
    details._selectedDates = _pickerStateDetails._selectedDates;
    details._selectedRange = _pickerStateDetails._selectedRange;
    details._selectedRanges = _pickerStateDetails._selectedRanges;
    details._view = _pickerStateDetails._view;
  }

  void _updatePickerViewStateDetails(_PickerStateArgs details) {
    _pickerStateDetails._currentDate = null;
    widget.getPickerStateValues(_pickerStateDetails);
    if (details._currentDate == null) {
      details._currentDate = _pickerStateDetails._currentDate;
    } else {
      _pickerStateDetails._currentDate = details._currentDate;
    }

    _pickerStateDetails._selectedDate = details._selectedDate;
    _pickerStateDetails._selectedDates = details._selectedDates;
    _pickerStateDetails._selectedRange = details._selectedRange;
    _pickerStateDetails._selectedRanges = details._selectedRanges;
    _pickerStateDetails._view = details._view;
    widget.updatePickerStateValues(_pickerStateDetails);
  }

  _PickerView _getView(List<DateTime> dates, dynamic key) {
    return _PickerView(
      widget.picker,
      widget.controller,
      dates,
      widget.width,
      widget.height,
      widget._datePickerTheme,
      _focusNode,
      key: key,
      getPickerStateDetails: (_PickerStateArgs details) {
        _getPickerViewStateDetails(details);
      },
      updatePickerStateDetails: (_PickerStateArgs details) {
        _updatePickerViewStateDetails(details);
      },
      isRtl: widget.isRtl,
    );
  }

  List<Widget> _addViews(BuildContext context) {
    _children = _children ?? <_PickerView>[];
    if (_children != null && _children.isEmpty) {
      _previousView = _getView(_previousViewVisibleDates, _previousViewKey);
      _currentView = _getView(_visibleDates, _currentViewKey);
      _nextView = _getView(_nextViewVisibleDates, _nextViewKey);

      _children.add(_previousView);
      _children.add(_currentView);
      _children.add(_nextView);
      return _children;
    }

    final dynamic previousView = _updateViews(
        _previousView, _previousView.visibleDates, _previousViewVisibleDates);
    final dynamic currentView =
        _updateViews(_currentView, _currentView.visibleDates, _visibleDates);
    final dynamic nextView =
        _updateViews(_nextView, _nextView.visibleDates, _nextViewVisibleDates);

    //// Update views while the all day view height differ from original height,
    //// else repaint the appointment painter while current child visible appointment not equals calendar visible appointment
    if (_previousView != previousView) {
      _previousView = previousView;
    }
    if (_currentView != currentView) {
      _currentView = currentView;
    }
    if (_nextView != nextView) {
      _nextView = nextView;
    }

    return _children;
  }

  // method to check and update the views and appointments on the swiping end
  Widget _updateViews(
      Widget view, List<DateTime> _viewDates, List<DateTime> _visibleDates) {
    final int index = _children.indexOf(view);
    // update the view with the visible dates on swiping end.
    if (_viewDates != _visibleDates) {
      view = _getView(_visibleDates, view.key);
      _children[index] = view;
    } // check and update the visible appointments in the view

    return view;
  }

  @override
  void dispose() {
    _animationController?.dispose();
    super.dispose();
  }

  void animationListener() {
    setState(() {
      _position = _animation.value;
    });
  }

  @override
  void didUpdateWidget(_PickerScrollView oldWidget) {
    if (widget.picker.navigationDirection !=
            oldWidget.picker.navigationDirection ||
        widget.width != oldWidget.width ||
        widget.picker.selectionMode != oldWidget.picker.selectionMode ||
        widget.height != oldWidget.height) {
      _position = 0;
      _children.clear();
    }

    if (oldWidget.picker.controller != widget.picker.controller) {
      _position = 0;
      _updateSelectedValuesWithMinMaxDate();
      _children.clear();
      _updateVisibleDates();
    }

    if (widget.isRtl != oldWidget.isRtl) {
      _position = 0;
      _children.clear();
      _updateVisibleDates();
    }

    if (widget.controller.view == DateRangePickerView.year &&
        (widget.picker.monthFormat != oldWidget.picker.monthFormat ||
            widget.picker.yearCellStyle != oldWidget.picker.yearCellStyle)) {
      _position = 0;
      _children.clear();
    }

    if (widget.picker.minDate != oldWidget.picker.minDate ||
        widget.picker.maxDate != oldWidget.picker.maxDate) {
      final DateTime _previousVisibleDate = _pickerStateDetails._currentDate;
      _pickerStateDetails._currentDate = null;
      widget.getPickerStateValues(_pickerStateDetails);
      if (!isSameDate(_pickerStateDetails._currentDate, _previousVisibleDate)) {
        _updateVisibleDates();
      }

      _position = 0;
      _updateSelectedValuesWithMinMaxDate();
      _children.clear();
    }

    if (widget.picker.enablePastDates != oldWidget.picker.enablePastDates) {
      _position = 0;
      _updateSelectedValuesWithMinMaxDate();
      _children.clear();
    }

    //// condition to check and update the view when the settings changed, it will check each and every property of settings
    //// to avoid unwanted repainting
    if (oldWidget.picker.monthViewSettings != widget.picker.monthViewSettings ||
        oldWidget.picker.monthCellStyle != widget.picker.monthCellStyle ||
        oldWidget.picker.monthViewSettings.selectionRadius !=
            widget.picker.monthViewSettings.selectionRadius ||
        oldWidget.picker.monthViewSettings.blackoutDates !=
            widget.picker.monthViewSettings.blackoutDates ||
        oldWidget.picker.monthViewSettings.specialDates !=
            widget.picker.monthViewSettings.specialDates ||
        oldWidget.picker.monthViewSettings.weekendDays !=
            widget.picker.monthViewSettings.weekendDays ||
        oldWidget.picker.selectionShape != widget.picker.selectionShape ||
        oldWidget.picker.todayHighlightColor !=
            widget.picker.todayHighlightColor ||
        oldWidget._locale != widget._locale) {
      _children.clear();
      _position = 0;
    }

    if (widget.controller.view == DateRangePickerView.month &&
        (oldWidget.picker.monthViewSettings.viewHeaderStyle !=
                widget.picker.monthViewSettings.viewHeaderStyle ||
            oldWidget.picker.monthViewSettings.viewHeaderHeight !=
                widget.picker.monthViewSettings.viewHeaderHeight ||
            widget.picker.monthViewSettings.showTrailingAndLeadingDates !=
                oldWidget
                    .picker.monthViewSettings.showTrailingAndLeadingDates)) {
      _children.clear();
      _position = 0;
    }

    if (widget.picker.monthViewSettings.numberOfWeeksInView !=
            oldWidget.picker.monthViewSettings.numberOfWeeksInView ||
        widget.picker.monthViewSettings.firstDayOfWeek !=
            oldWidget.picker.monthViewSettings.firstDayOfWeek) {
      _updateVisibleDates();
      _position = 0;
    }

    if (oldWidget.picker.controller != widget.picker.controller ||
        widget.picker.controller == null) {
      super.didUpdateWidget(oldWidget);
      return;
    }

    if (oldWidget.picker.controller.displayDate !=
            widget.picker.controller.displayDate ||
        !isSameDate(
            _pickerStateDetails._currentDate, widget.controller.displayDate)) {
      _pickerStateDetails._currentDate = widget.picker.controller.displayDate;
      _updateVisibleDates();
    }

    if (widget.picker.selectionMode == DateRangePickerSelectionMode.single &&
        (oldWidget.picker.controller.selectedDate !=
                widget.picker.controller.selectedDate ||
            !isSameDate(_pickerStateDetails._selectedDate,
                widget.controller.selectedDate))) {
      _pickerStateDetails._selectedDate = widget.picker.controller.selectedDate;
      _drawSelection();
      _position = 0;
    }

    if (widget.picker.selectionMode == DateRangePickerSelectionMode.multiple &&
        (oldWidget.picker.controller.selectedDates !=
                widget.picker.controller.selectedDates ||
            !_isDateCollectionEquals(_pickerStateDetails._selectedDates,
                widget.picker.controller.selectedDates))) {
      _pickerStateDetails._selectedDates =
          widget.picker.controller.selectedDates;
      _drawSelection();
      _position = 0;
    }

    if (widget.picker.selectionMode == DateRangePickerSelectionMode.range &&
        (oldWidget.picker.controller.selectedRange !=
                widget.picker.controller.selectedRange ||
            !_isRangeEquals(_pickerStateDetails._selectedRange,
                widget.picker.controller.selectedRange))) {
      _pickerStateDetails._selectedRange =
          widget.picker.controller.selectedRange;
      _drawSelection();
      _position = 0;
    }

    if (widget.picker.selectionMode ==
            DateRangePickerSelectionMode.multiRange &&
        (oldWidget.picker.controller.selectedRanges !=
                widget.picker.controller.selectedRanges ||
            !_isDateRangesEquals(_pickerStateDetails._selectedRanges,
                widget.picker.controller.selectedRanges))) {
      _pickerStateDetails._selectedRanges =
          widget.picker.controller.selectedRanges;
      _drawSelection();
      _position = 0;
    }

    if (widget.picker.controller.view != oldWidget.picker.controller.view ||
        _pickerStateDetails._view != widget.controller._view) {
      _pickerStateDetails._view = widget.controller._view;
      _position = 0;
      _children.clear();
      _updateVisibleDates();
    }

    super.didUpdateWidget(oldWidget);
  }

  void _updateSelectedValuesWithMinMaxDate() {
    widget.getPickerStateValues(_pickerStateDetails);
    if (widget.picker.selectionMode == DateRangePickerSelectionMode.single &&
        _pickerStateDetails._selectedDate != null) {
      if (!_isEnabledDate(widget.picker.minDate, widget.picker.maxDate,
          widget.picker.enablePastDates, _pickerStateDetails._selectedDate)) {
        _pickerStateDetails._selectedDate = null;
        widget.updatePickerStateValues(_pickerStateDetails);
      }
    } else if (widget.picker.selectionMode ==
            DateRangePickerSelectionMode.multiple &&
        _pickerStateDetails._selectedDates != null &&
        _pickerStateDetails._selectedDates.isNotEmpty) {
      final List<int> indexList = <int>[];
      for (int i = 0; i < _pickerStateDetails._selectedDates.length; i++) {
        if (!_isEnabledDate(
            widget.picker.minDate,
            widget.picker.maxDate,
            widget.picker.enablePastDates,
            _pickerStateDetails._selectedDates[i])) {
          indexList.add(i);
        }
      }

      if (indexList != null && indexList.isNotEmpty) {
        for (int i = indexList.length - 1; i >= 0; i--) {
          _pickerStateDetails._selectedDates.removeAt(indexList[i]);
        }

        widget.updatePickerStateValues(_pickerStateDetails);
      }
    } else if (widget.picker.selectionMode ==
            DateRangePickerSelectionMode.range &&
        _pickerStateDetails._selectedRange != null) {
      if ((_pickerStateDetails._selectedRange.startDate != null &&
              !_isEnabledDate(
                  widget.picker.minDate,
                  widget.picker.maxDate,
                  widget.picker.enablePastDates,
                  _pickerStateDetails._selectedRange.startDate)) ||
          (_pickerStateDetails._selectedRange.endDate != null &&
              !_isEnabledDate(
                  widget.picker.minDate,
                  widget.picker.maxDate,
                  widget.picker.enablePastDates,
                  _pickerStateDetails._selectedRange.endDate))) {
        _pickerStateDetails._selectedRange = null;
      }

      widget.updatePickerStateValues(_pickerStateDetails);
    } else if (widget.picker.selectionMode ==
            DateRangePickerSelectionMode.multiRange &&
        _pickerStateDetails._selectedRanges != null &&
        _pickerStateDetails._selectedRanges.isNotEmpty) {
      final List<int> indexList = <int>[];
      for (int i = 0; i < _pickerStateDetails._selectedRanges.length; i++) {
        final PickerDateRange _range = _pickerStateDetails._selectedRanges[i];
        if ((_range.startDate != null &&
                !_isEnabledDate(widget.picker.minDate, widget.picker.maxDate,
                    widget.picker.enablePastDates, _range.startDate)) ||
            (_range.endDate != null &&
                !_isEnabledDate(widget.picker.minDate, widget.picker.maxDate,
                    widget.picker.enablePastDates, _range.endDate))) {
          indexList.add(i);
        }
      }

      if (indexList != null && indexList.isNotEmpty) {
        for (int i = indexList.length - 1; i >= 0; i--) {
          _pickerStateDetails._selectedRanges.removeAt(indexList[i]);
        }

        widget.updatePickerStateValues(_pickerStateDetails);
      }
    }
  }

  void _moveToNextViewWithAnimation() {
    // Resets the controller to forward it again, the animation will forward only from the dismissed state
    if (_animationController.isCompleted || _animationController.isDismissed) {
      _animationController.reset();
    } else {
      return;
    }

    _updateSelection();
    if (widget.picker.navigationDirection ==
        DateRangePickerNavigationDirection.vertical) {
      // update the bottom to top swiping
      _tween.begin = 0;
      _tween.end = -widget.height;
    } else {
      // update the right to left swiping
      _tween.begin = 0;
      _tween.end = -widget.width;
    }

    _animationController.duration = const Duration(milliseconds: 500);
    _animationController
        .forward()
        .then<dynamic>((dynamic value) => _updateNextView());

    /// updates the current view visible dates when the view swiped
    _updateCurrentViewVisibleDates(isNextView: true);
  }

  // ignore: unused_element
  void _moveToPreviousViewWithAnimation() {
    // Resets the controller to backward it again, the animation will backward only from the dismissed state
    if (_animationController.isCompleted || _animationController.isDismissed) {
      _animationController.reset();
    } else {
      return;
    }

    _updateSelection();
    if (widget.picker.navigationDirection ==
        DateRangePickerNavigationDirection.vertical) {
      // update the top to bottom swiping
      _tween.begin = 0;
      _tween.end = widget.height;
    } else {
      // update the left to right swiping
      _tween.begin = 0;
      _tween.end = widget.width;
    }

    _animationController.duration = const Duration(milliseconds: 500);
    _animationController
        .forward()
        .then<dynamic>((dynamic value) => _updatePreviousView());

    /// updates the current view visible dates when the view swiped.
    _updateCurrentViewVisibleDates();
  }

  void _updateSelection() {
    if (widget.controller.view != DateRangePickerView.month) {
      return;
    }

    widget.getPickerStateValues(_pickerStateDetails);
    for (int i = 0; i < _children.length; i++) {
      if (i == _currentChildIndex) {
        continue;
      }

      final _PickerViewState _viewState = _getCurrentViewState(i);
      _viewState._monthView._updateSelection(_pickerStateDetails);
    }
  }

  void _drawSelection() {
    if (widget.controller.view != DateRangePickerView.month ||
        _children.isEmpty) {
      return;
    }

    final _PickerViewState _viewState =
        _getCurrentViewState(_currentChildIndex);
    _viewState._monthView._updateSelection(_pickerStateDetails);
  }

  _PickerViewState _getCurrentViewState(int index) {
    final _PickerViewState previousViewState = _previousViewKey.currentState;
    final _PickerViewState currentViewState = _currentViewKey.currentState;
    final _PickerViewState nextViewState = _nextViewKey.currentState;

    _PickerViewState _viewState = previousViewState;
    if (index == 1) {
      _viewState = currentViewState;
    } else if (index == 2) {
      _viewState = nextViewState;
    }

    return _viewState;
  }

  /// Updates the current view visible dates for calendar in the swiping end
  void _updateCurrentViewVisibleDates({bool isNextView = false}) {
    if (isNextView) {
      if (_currentChildIndex == 0) {
        _currentViewVisibleDates = _visibleDates;
      } else if (_currentChildIndex == 1) {
        _currentViewVisibleDates = _nextViewVisibleDates;
      } else {
        _currentViewVisibleDates = _previousViewVisibleDates;
      }
    } else {
      if (_currentChildIndex == 0) {
        _currentViewVisibleDates = _nextViewVisibleDates;
      } else if (_currentChildIndex == 1) {
        _currentViewVisibleDates = _previousViewVisibleDates;
      } else {
        _currentViewVisibleDates = _visibleDates;
      }
    }

    _pickerStateDetails._currentViewVisibleDates = _currentViewVisibleDates;
    _pickerStateDetails._currentDate = _currentViewVisibleDates[0];
    if (widget.controller.view == DateRangePickerView.month &&
        widget.picker.monthViewSettings.numberOfWeeksInView == 6) {
      final DateTime _date =
          _currentViewVisibleDates[_currentViewVisibleDates.length ~/ 2];
      _pickerStateDetails._currentDate = DateTime(_date.year, _date.month, 1);
    }

    widget.updatePickerStateValues(_pickerStateDetails);
  }

  dynamic _updateNextView() {
    if (!_animationController.isCompleted) {
      return;
    }

    _updateNextViewVisibleDates();

    if (_currentChildIndex == 0) {
      _currentChildIndex = 1;
    } else if (_currentChildIndex == 1) {
      _currentChildIndex = 2;
    } else if (_currentChildIndex == 2) {
      _currentChildIndex = 0;
    }

    /// set state called to call the build method to fix the date doesn't update
    /// properly issue on web, in Andriod and iOS the build method called
    /// automatically when the animation ends but in web it doesn't work on that
    /// way, hence we have manually called the build method by adding setstate
    /// and i have logged and issue in framework once i got the solution will
    /// remove this setstate
    if (kIsWeb) {
      setState(() {});
    }

    _resetPosition();
  }

  dynamic _updatePreviousView() {
    if (!_animationController.isCompleted) {
      return;
    }

    _updatePreviousViewVisibleDates();

    if (_currentChildIndex == 0) {
      _currentChildIndex = 2;
    } else if (_currentChildIndex == 1) {
      _currentChildIndex = 0;
    } else if (_currentChildIndex == 2) {
      _currentChildIndex = 1;
    }

    /// set state called to call the build method to fix the date doesn't update
    /// properly issue on web, in Andriod and iOS the build method called
    /// automatically when the animation ends but in web it doesn't work on that
    /// way, hence we have manually called the build method by adding setstate
    /// and i have logged and issue in framework once i got the solution will
    /// remove this setstate
    if (kIsWeb) {
      setState(() {});
    }

    _resetPosition();
  }

  // resets position to zero on the swipe end to avoid the unwanted date updates.
  void _resetPosition() {
    SchedulerBinding.instance.addPostFrameCallback((_) {
      if (_position.abs() == widget.width || _position.abs() == widget.height) {
        _position = 0;
      }
    });
  }

  dynamic _onKeyDown(RawKeyEvent event) {
    if (event.runtimeType != RawKeyDownEvent) {
      return;
    }

    /// Ctrl + and Ctrl - used by browser to zoom the page, hence as referred the
    /// EJ2 scheduler, we have used alt + numeric to switch between views in
    /// datepicker web
    if (event.isAltPressed) {
      if (event.physicalKey == PhysicalKeyboardKey.digit1) {
        _pickerStateDetails._view = DateRangePickerView.month;
      } else if (event.physicalKey == PhysicalKeyboardKey.digit2) {
        _pickerStateDetails._view = DateRangePickerView.year;
      } else if (event.physicalKey == PhysicalKeyboardKey.digit3) {
        _pickerStateDetails._view = DateRangePickerView.decade;
      } else if (event.physicalKey == PhysicalKeyboardKey.digit4) {
        _pickerStateDetails._view = DateRangePickerView.century;
      }

      widget.updatePickerStateValues(_pickerStateDetails);
      return;
    }

    if (widget.controller.view != DateRangePickerView.month) {
      return;
    }

    _PickerViewState _currentVisibleViewState;
    _PickerView _currentVisibleView;
    if (_currentChildIndex == 0) {
      _currentVisibleViewState = _previousViewKey.currentState;
      _currentVisibleView = _previousView;
    } else if (_currentChildIndex == 1) {
      _currentVisibleViewState = _currentViewKey.currentState;
      _currentVisibleView = _currentView;
    } else if (_currentChildIndex == 2) {
      _currentVisibleViewState = _nextViewKey.currentState;
      _currentVisibleView = _nextView;
    }

    if (_pickerStateDetails._selectedDate != null ||
        (_pickerStateDetails._selectedDates != null &&
            _pickerStateDetails._selectedDates.isNotEmpty) ||
        _pickerStateDetails._selectedRange != null ||
        (_pickerStateDetails._selectedRanges != null &&
            _pickerStateDetails._selectedRanges.isNotEmpty)) {
      final DateTime selectedDate = _updateSelectedDate(
          event, _currentVisibleViewState, _currentVisibleView);

      if (_isDateWithInVisibleDates(_currentVisibleView.visibleDates,
              widget.picker.monthViewSettings.blackoutDates, selectedDate) ||
          !_isEnabledDate(widget.picker.minDate, widget.picker.maxDate,
              widget.picker.enablePastDates, selectedDate)) {
        return;
      }

      if (!_isDateAsCurrentMonthDate(
          _currentVisibleView
              .visibleDates[_currentVisibleView.visibleDates.length ~/ 2],
          widget.picker.monthViewSettings.numberOfWeeksInView,
          widget.picker.monthViewSettings.showTrailingAndLeadingDates,
          selectedDate)) {
        if (selectedDate.month ==
            getNextMonthDate(_currentVisibleView
                    .visibleDates[_currentVisibleView.visibleDates.length ~/ 2])
                .month) {
          _moveToNextViewWithAnimation();
        } else {
          _moveToPreviousViewWithAnimation();
        }
      }

      _currentVisibleViewState._drawSelection(selectedDate);
      if (widget.picker.selectionMode == DateRangePickerSelectionMode.single) {
        _pickerStateDetails._selectedDate = selectedDate;
      } else if (widget.picker.selectionMode ==
          DateRangePickerSelectionMode.multiple) {
        _pickerStateDetails._selectedDates.add(selectedDate);
      } else if (widget.picker.selectionMode ==
          DateRangePickerSelectionMode.range) {
        if (_pickerStateDetails._selectedRange != null &&
            _pickerStateDetails._selectedRange.startDate != null &&
            (_pickerStateDetails._selectedRange.endDate == null ||
                isSameDate(_pickerStateDetails._selectedRange.startDate,
                    _pickerStateDetails._selectedRange.endDate))) {
          _pickerStateDetails._selectedRange = PickerDateRange(
              _pickerStateDetails._selectedRange.startDate, selectedDate);
        } else {
          _pickerStateDetails._selectedRange =
              PickerDateRange(selectedDate, null);
        }
      }
      widget.updatePickerStateValues(_pickerStateDetails);
      _updateSelection();
    }
  }

  DateTime _updateSelectedDate(RawKeyEvent event, _PickerViewState currentState,
      _PickerView currentView) {
    if (widget.picker.selectionMode == DateRangePickerSelectionMode.single) {
      if (event.physicalKey == PhysicalKeyboardKey.arrowRight) {
        if (isSameDate(_pickerStateDetails._selectedDate,
            currentView.visibleDates[currentView.visibleDates.length - 1])) {
          _moveToNextViewWithAnimation();
        }

        return _pickerStateDetails._selectedDate.add(const Duration(days: 1));
      } else if (event.physicalKey == PhysicalKeyboardKey.arrowLeft) {
        if (isSameDate(
            _pickerStateDetails._selectedDate, currentView.visibleDates[0])) {
          _moveToPreviousViewWithAnimation();
        }

        return _pickerStateDetails._selectedDate
            .subtract(const Duration(days: 1));
      } else if (event.physicalKey == PhysicalKeyboardKey.arrowUp) {
        return _pickerStateDetails._selectedDate
            .subtract(const Duration(days: 7));
      } else if (event.physicalKey == PhysicalKeyboardKey.arrowDown) {
        return _pickerStateDetails._selectedDate.add(const Duration(days: 7));
      }
    } else if (widget.picker.selectionMode !=
        DateRangePickerSelectionMode.multiRange) {
      if (event.isShiftPressed &&
          event.physicalKey == PhysicalKeyboardKey.arrowRight) {
        if (widget.picker.selectionMode ==
            DateRangePickerSelectionMode.multiple) {
          return _pickerStateDetails
              ._selectedDates[_pickerStateDetails._selectedDates.length - 1]
              .add(const Duration(days: 1));
        } else {
          return _pickerStateDetails._selectedRange.startDate
              .add(const Duration(days: 1));
        }
      } else if (event.isShiftPressed &&
          event.physicalKey == PhysicalKeyboardKey.arrowLeft) {
        if (widget.picker.selectionMode ==
            DateRangePickerSelectionMode.multiple) {
          return _pickerStateDetails
              ._selectedDates[_pickerStateDetails._selectedDates.length - 1]
              .add(const Duration(days: -1));
        } else {
          return _pickerStateDetails._selectedRange.startDate
              .add(const Duration(days: -1));
        }
      } else if (event.isShiftPressed &&
          event.physicalKey == PhysicalKeyboardKey.arrowUp) {
        if (widget.picker.selectionMode ==
            DateRangePickerSelectionMode.multiple) {
          return _pickerStateDetails
              ._selectedDates[_pickerStateDetails._selectedDates.length - 1]
              .add(const Duration(days: -7));
        } else {
          return _pickerStateDetails._selectedRange.startDate
              .add(const Duration(days: -7));
        }
      } else if (event.isShiftPressed &&
          event.physicalKey == PhysicalKeyboardKey.arrowDown) {
        if (widget.picker.selectionMode ==
            DateRangePickerSelectionMode.multiple) {
          return _pickerStateDetails
              ._selectedDates[_pickerStateDetails._selectedDates.length - 1]
              .add(const Duration(days: 7));
        } else {
          return _pickerStateDetails._selectedRange.startDate
              .add(const Duration(days: 7));
        }
      }
    }

    return null;
  }

  @override
  Widget build(BuildContext context) {
    double _leftPosition, _rightPosition, _topPosition, _bottomPosition;
    if (widget.picker.navigationDirection ==
        DateRangePickerNavigationDirection.horizontal) {
      _leftPosition = _leftPosition ?? -widget.width;
      _rightPosition = _rightPosition ?? -widget.width;
      _topPosition = 0;
      _bottomPosition = 0;
    } else {
      _leftPosition = 0;
      _rightPosition = 0;
      _topPosition = _topPosition ?? -widget.height;
      _bottomPosition = _bottomPosition ?? -widget.height;
    }

    return Stack(
      children: <Widget>[
        Positioned(
          left: _leftPosition,
          right: _rightPosition,
          bottom: _bottomPosition,
          top: _topPosition,
          child: GestureDetector(
            child: RawKeyboardListener(
              focusNode: _focusNode,
              onKey: _onKeyDown,
              child: CustomScrollViewerLayout(
                  _addViews(context),
                  widget.picker.navigationDirection ==
                          DateRangePickerNavigationDirection.horizontal
                      ? CustomScrollDirection.horizontal
                      : CustomScrollDirection.vertical,
                  _position,
                  _currentChildIndex),
            ),
            onHorizontalDragStart: (DragStartDetails details) {
              if (widget.picker.navigationDirection ==
                  DateRangePickerNavigationDirection.horizontal) {
                _onHorizontalStart(details);
              }
            },
            onHorizontalDragUpdate: (DragUpdateDetails details) {
              if (widget.picker.navigationDirection ==
                  DateRangePickerNavigationDirection.horizontal) {
                _onHorizontalUpdate(details);
              }
            },
            onHorizontalDragEnd: (DragEndDetails details) {
              if (widget.picker.navigationDirection ==
                  DateRangePickerNavigationDirection.horizontal) {
                _onHorizontalEnd(details);
              }
            },
            onVerticalDragStart: (DragStartDetails details) {
              if (widget.picker.navigationDirection ==
                  DateRangePickerNavigationDirection.vertical) {
                _onVerticalStart(details);
              }
            },
            onVerticalDragUpdate: (DragUpdateDetails details) {
              if (widget.picker.navigationDirection ==
                  DateRangePickerNavigationDirection.vertical) {
                _onVerticalUpdate(details);
              }
            },
            onVerticalDragEnd: (DragEndDetails details) {
              if (widget.picker.navigationDirection ==
                  DateRangePickerNavigationDirection.vertical) {
                _onVerticalEnd(details);
              }
            },
          ),
        )
      ],
    );
  }

  void _onHorizontalStart(DragStartDetails dragStartDetails) {
    if (widget.picker.navigationDirection ==
        DateRangePickerNavigationDirection.horizontal) {
      _scrollStartPosition = dragStartDetails.globalPosition.dx;
      _updateSelection();
    }
  }

  void _onHorizontalUpdate(DragUpdateDetails dragUpdateDetails) {
    if (widget.picker.navigationDirection ==
        DateRangePickerNavigationDirection.horizontal) {
      final double difference =
          dragUpdateDetails.globalPosition.dx - _scrollStartPosition;
      if (difference < 0 &&
          !_canMoveToNextViewRtl(
              widget.controller.view,
              widget.picker.monthViewSettings.numberOfWeeksInView,
              widget.picker.minDate,
              widget.picker.maxDate,
              _currentViewVisibleDates,
              widget.isRtl)) {
        return;
      } else if (difference > 0 &&
          !_canMoveToPreviousViewRtl(
              widget.controller.view,
              widget.picker.monthViewSettings.numberOfWeeksInView,
              widget.picker.minDate,
              widget.picker.maxDate,
              _currentViewVisibleDates,
              widget.isRtl)) {
        return;
      }

      _position = difference;
      setState(() {});
    }
  }

  void _onHorizontalEnd(DragEndDetails dragEndDetails) {
    if (widget.picker.navigationDirection !=
        DateRangePickerNavigationDirection.horizontal) {
      return;
    }

    _position ??= 0;
    // condition to check and update the right to left swiping
    if (-_position >= widget.width / 2) {
      _tween.begin = _position;
      _tween.end = -widget.width;

      // Resets the controller to forward it again, the animation will forward only from the dismissed state
      if (_animationController.isCompleted && _position != _tween.end) {
        _animationController.reset();
      }

      _animationController.duration = const Duration(milliseconds: 250);
      _animationController
          .forward()
          .then<dynamic>((dynamic value) => _updateNextView());

      /// updates the current view visible dates when the view swiped in
      /// right to left direction
      _updateCurrentViewVisibleDates(isNextView: true);
    }
    // fling the view from right to left
    else if (-dragEndDetails.velocity.pixelsPerSecond.dx > widget.width) {
      if (!_canMoveToNextViewRtl(
          widget.controller.view,
          widget.picker.monthViewSettings.numberOfWeeksInView,
          widget.picker.minDate,
          widget.picker.maxDate,
          _currentViewVisibleDates,
          widget.isRtl)) {
        _position = 0;
        setState(() {});
        return;
      }

      _tween.begin = _position;
      _tween.end = -widget.width;

      // Resets the controller to forward it again, the animation will forward only from the dismissed state
      if (_animationController.isCompleted && _position != _tween.end)
        _animationController.reset();

      _animationController.duration = const Duration(milliseconds: 250);
      _animationController
          .fling(velocity: 5.0, animationBehavior: AnimationBehavior.normal)
          .then<dynamic>((dynamic value) => _updateNextView());

      /// updates the current view visible dates when fling the view in
      /// right to left direction
      _updateCurrentViewVisibleDates(isNextView: true);
    }
    // condition to check and update the left to right swiping
    else if (_position >= widget.width / 2) {
      _tween.begin = _position;
      _tween.end = widget.width;

      // Resets the controller to forward it again, the animation will forward only from the dismissed state
      if (_animationController.isCompleted || _position != _tween.end) {
        _animationController.reset();
      }

      _animationController.duration = const Duration(milliseconds: 250);
      _animationController
          .forward()
          .then<dynamic>((dynamic value) => _updatePreviousView());

      /// updates the current view visible dates when the view swiped in
      /// left to right direction
      _updateCurrentViewVisibleDates();
    }
    // fling the view from left to right
    else if (dragEndDetails.velocity.pixelsPerSecond.dx > widget.width) {
      if (!_canMoveToPreviousViewRtl(
          widget.controller.view,
          widget.picker.monthViewSettings.numberOfWeeksInView,
          widget.picker.minDate,
          widget.picker.maxDate,
          _currentViewVisibleDates,
          widget.isRtl)) {
        _position = 0;
        setState(() {});
        return;
      }

      _tween.begin = _position;
      _tween.end = widget.width;

      // Resets the controller to forward it again, the animation will forward only from the dismissed state
      if (_animationController.isCompleted && _position != _tween.end)
        _animationController.reset();

      _animationController.duration = const Duration(milliseconds: 250);
      _animationController
          .fling(velocity: 5.0, animationBehavior: AnimationBehavior.normal)
          .then<dynamic>((dynamic value) => _updatePreviousView());

      /// updates the current view visible dates when fling the view in
      /// left to right direction
      _updateCurrentViewVisibleDates();
    }
    // condition to check and revert the right to left swiping
    else if (_position.abs() <= widget.width / 2) {
      _tween.begin = _position;
      _tween.end = 0.0;

      // Resets the controller to forward it again, the animation will forward only from the dismissed state
      if (_animationController.isCompleted && _position != _tween.end) {
        _animationController.reset();
      }

      _animationController.duration = const Duration(milliseconds: 250);
      _animationController.forward();
    }
  }

  void _onVerticalStart(DragStartDetails dragStartDetails) {
    if (widget.picker.navigationDirection ==
        DateRangePickerNavigationDirection.vertical) {
      _scrollStartPosition = dragStartDetails.globalPosition.dy;
    }
  }

  void _onVerticalUpdate(DragUpdateDetails dragUpdateDetails) {
    if (widget.picker.navigationDirection ==
        DateRangePickerNavigationDirection.vertical) {
      final double difference =
          dragUpdateDetails.globalPosition.dy - _scrollStartPosition;
      if (difference < 0 &&
          !_canMoveToNextView(
              widget.controller.view,
              widget.picker.monthViewSettings.numberOfWeeksInView,
              widget.picker.maxDate,
              _currentViewVisibleDates)) {
        return;
      } else if (difference > 0 &&
          !_canMoveToPreviousView(
              widget.controller.view,
              widget.picker.monthViewSettings.numberOfWeeksInView,
              widget.picker.minDate,
              _currentViewVisibleDates)) {
        return;
      }

      _position = difference;
      setState(() {});
    }
  }

  void _onVerticalEnd(DragEndDetails dragEndDetails) {
    if (widget.picker.navigationDirection !=
        DateRangePickerNavigationDirection.vertical) {
      return;
    }

    _position ??= 0;
    // condition to check and update the bottom to top swiping
    if (-_position >= widget.height / 2) {
      _tween.begin = _position;
      _tween.end = -widget.height;

      // Resets the controller to forward it again, the animation will forward only from the dismissed state
      if (_animationController.isCompleted || _position != _tween.end) {
        _animationController.reset();
      }

      _animationController.duration = const Duration(milliseconds: 250);
      _animationController
          .forward()
          .then<dynamic>((dynamic value) => _updateNextView());

      /// updates the current view visible dates when the view swiped in
      /// bottom to top direction
      _updateCurrentViewVisibleDates(isNextView: true);
    }
    // fling the view to bottom to top
    else if (-dragEndDetails.velocity.pixelsPerSecond.dy > widget.height) {
      if (!_canMoveToNextView(
          widget.controller.view,
          widget.picker.monthViewSettings.numberOfWeeksInView,
          widget.picker.maxDate,
          _currentViewVisibleDates)) {
        _position = 0;
        setState(() {});
        return;
      }
      _tween.begin = _position;
      _tween.end = -widget.height;

      // Resets the controller to forward it again, the animation will forward only from the dismissed state
      if (_animationController.isCompleted || _position != _tween.end) {
        _animationController.reset();
      }

      _animationController.duration = const Duration(milliseconds: 250);
      _animationController
          .fling(velocity: 5.0, animationBehavior: AnimationBehavior.normal)
          .then<dynamic>((dynamic value) => _updateNextView());

      /// updates the current view visible dates when fling the view in
      /// bottom to top direction
      _updateCurrentViewVisibleDates(isNextView: true);
    }
    // condition to check and update the top to bottom swiping
    else if (_position >= widget.height / 2) {
      _tween.begin = _position;
      _tween.end = widget.height;

      // Resets the controller to forward it again, the animation will forward only from the dismissed state
      if (_animationController.isCompleted || _position != _tween.end) {
        _animationController.reset();
      }

      _animationController.duration = const Duration(milliseconds: 250);
      _animationController
          .forward()
          .then<dynamic>((dynamic value) => _updatePreviousView());

      /// updates the current view visible dates when the view swiped in
      /// top to bottom direction
      _updateCurrentViewVisibleDates();
    }
    // fling the view to top to bottom
    else if (dragEndDetails.velocity.pixelsPerSecond.dy > widget.height) {
      if (!_canMoveToPreviousView(
          widget.controller.view,
          widget.picker.monthViewSettings.numberOfWeeksInView,
          widget.picker.minDate,
          _currentViewVisibleDates)) {
        _position = 0;
        setState(() {});
        return;
      }

      _tween.begin = _position;
      _tween.end = widget.height;

      // Resets the controller to forward it again, the animation will forward only from the dismissed state
      if (_animationController.isCompleted || _position != _tween.end) {
        _animationController.reset();
      }

      _animationController.duration = const Duration(milliseconds: 250);
      _animationController
          .fling(velocity: 5.0, animationBehavior: AnimationBehavior.normal)
          .then<dynamic>((dynamic value) => _updatePreviousView());

      /// updates the current view visible dates when fling the view in
      /// top to bottom direction
      _updateCurrentViewVisibleDates();
    }
    // condition to check and revert the bottom to top swiping
    else if (_position.abs() <= widget.height / 2) {
      _tween.begin = _position;
      _tween.end = 0.0;

      // Resets the controller to forward it again, the animation will forward only from the dismissed state
      if (_animationController.isCompleted || _position != _tween.end)
        _animationController.reset();

      _animationController.duration = const Duration(milliseconds: 250);
      _animationController.forward();
    }
  }
}
