part of datepicker;

int _getRtlIndex(int count, int index) {
  return count - index - 1;
}

List<dynamic> _cloneList(List<dynamic> value) {
  if (value == null || value.isEmpty) {
    return null;
  }

  return value.sublist(0);
}

double _getCellRadius(
    double selectionRadius, double maxXRadius, double maxYRadius) {
  final double radius = maxXRadius > maxYRadius
      ? maxYRadius - _IMonthViewPainter._selectionPadding
      : maxXRadius - _IMonthViewPainter._selectionPadding;

  if (selectionRadius == null) {
    return radius;
  }

  return radius > selectionRadius ? selectionRadius : radius;
}

List<int> _getSelectedRangeIndex(
    DateTime startDate, DateTime endDate, List<DateTime> visibleDates) {
  int startIndex = -1;
  int endIndex = -1;
  final List<int> _selectedIndex = <int>[];
  if (startDate != null && startDate.isAfter(endDate)) {
    final DateTime temp = startDate;
    startDate = endDate;
    endDate = temp;
  }

  final DateTime viewStartDate = visibleDates[0];
  final DateTime viewEndDate = visibleDates[visibleDates.length - 1];
  if (startDate != null) {
    if (viewStartDate.isAfter(startDate) && viewStartDate.isBefore(endDate)) {
      startIndex = -1;
    } else {
      startIndex = _getSelectedIndex(startDate, visibleDates);
    }
  }

  if (endDate != null) {
    if (viewEndDate.isAfter(startDate) && viewEndDate.isBefore(endDate)) {
      endIndex = visibleDates.length;
    } else {
      endIndex = _getSelectedIndex(endDate, visibleDates);
    }
  }

  //// If some range end date as null then it end index is start index.
  if (startIndex != -1 && endIndex == -1) {
    endIndex = startIndex;
  }

  if (startIndex > endIndex) {
    final int temp = startIndex;
    startIndex = endIndex;
    endIndex = temp;
  }

  for (int i = startIndex; i <= endIndex; i++) {
    _selectedIndex.add(i);
  }

  return _selectedIndex;
}

int _getSelectedIndex(DateTime date, List<DateTime> visibleDates) {
  for (int i = 0; i < visibleDates.length; i++) {
    if (isSameDate(visibleDates[i], date)) {
      return i;
    }
  }

  return -1;
}

void _drawCircleSelection(
    Canvas canvas, double x, double y, double radius, Paint selectionPainter) {
  canvas.drawCircle(Offset(x, y), radius, selectionPainter);
}

void _drawFillSelection(Canvas canvas, double x, double y, double width,
    double height, Paint selectionPainter) {
  const double padding = 1;
  canvas.drawRRect(
      RRect.fromRectAndRadius(
          Rect.fromLTRB(x + padding, y + padding, x + width - padding,
              y + height - padding),
          Radius.circular(height / 4 > 10 ? 10 : height / 4)),
      selectionPainter);
}

void _drawStartRangeFillSelection(Canvas canvas, double x, double y,
    double width, double height, Paint selectionPainter) {
  const double padding = 1;
  final double cornerRadius = height / 4 > 10 ? 10 : height / 4;
  canvas.drawRRect(
      RRect.fromRectAndCorners(
          Rect.fromLTRB(
              x + padding, y + padding, x + width, y + height - padding),
          bottomLeft: Radius.circular(cornerRadius),
          topLeft: Radius.circular(cornerRadius)),
      selectionPainter);
}

void _drawEndRangeFillSelection(Canvas canvas, double x, double y, double width,
    double height, Paint selectionPainter) {
  const double padding = 1;
  final double cornerRadius = height / 4 > 10 ? 10 : height / 4;
  canvas.drawRRect(
      RRect.fromRectAndCorners(
          Rect.fromLTRB(
              x, y + padding, x + width - padding, y + height - padding),
          bottomRight: Radius.circular(cornerRadius),
          topRight: Radius.circular(cornerRadius)),
      selectionPainter);
}

void _drawStartEndRangeCircleSelection(Canvas canvas, double x, double y,
    double radius, Rect rect, Paint selectionPainter, Color color) {
  canvas.drawRect(rect, selectionPainter);
  selectionPainter.isAntiAlias = true;
  canvas.drawCircle(Offset(x, y), radius, selectionPainter);
  selectionPainter.color = color;
  canvas.drawCircle(Offset(x, y), radius - _IMonthViewPainter._selectionPadding,
      selectionPainter);
}

void _drawRectRangeSelection(Canvas canvas, double left, double top,
    double right, double bottom, Paint selectionPainter) {
  canvas.drawRect(Rect.fromLTRB(left, top, right, bottom), selectionPainter);
}

void _drawMonthCellsAndSelection(Canvas canvas, Size size,
    _IMonthViewPainter monthViewPainter, double cellWidth, double cellHeight) {
  canvas.clipRect(Rect.fromLTWH(0, 0, size.width, size.height));
  double _xPosition, _yPosition;
  _xPosition = 0;
  _yPosition = 0;
  monthViewPainter._textPainter = monthViewPainter._textPainter ??
      TextPainter(
          textAlign: TextAlign.center,
          textDirection: TextDirection.ltr,
          textWidthBasis: TextWidthBasis.longestLine);
  TextStyle _textStyle = monthViewPainter.cellStyle.textStyle;
  final DateTime _currentDate = monthViewPainter
      .visibleDates[(monthViewPainter.visibleDates.length / 2).truncate()];
  final int _nextMonth = getNextMonthDate(_currentDate).month;
  final int _previousMonth = getPreviousMonthDate(_currentDate).month;
  final DateTime _today = DateTime.now();
  bool _isCurrentDate;

  final TextStyle _currentDatesTextStyle =
      monthViewPainter.cellStyle.textStyle ??
          monthViewPainter.datePickerTheme.activeDatesTextStyle;
  final TextStyle _leadingDatesTextStyle =
      monthViewPainter.cellStyle.leadingDatesTextStyle ??
          monthViewPainter.datePickerTheme.leadingDatesTextStyle;
  final TextStyle _trailingDatesTextStyle =
      monthViewPainter.cellStyle.trailingDatesTextStyle ??
          monthViewPainter.datePickerTheme.trailingDatesTextStyle;
  final TextStyle _disabledDatesTextStyle =
      monthViewPainter.cellStyle.disabledDatesTextStyle ??
          monthViewPainter.datePickerTheme.disabledDatesTextStyle;
  final TextStyle _specialDatesTextStyle =
      monthViewPainter.cellStyle.specialDatesTextStyle ??
          monthViewPainter.datePickerTheme.specialDatesTextStyle;
  final TextStyle _selectionTextStyle =
      monthViewPainter.cellStyle.selectionTextStyle ??
          monthViewPainter.datePickerTheme.selectionTextStyle;
  final TextStyle _selectedRangeTextStyle =
      monthViewPainter.cellStyle.rangeTextStyle ??
          monthViewPainter.datePickerTheme.rangeSelectionTextStyle;

  Decoration _dateDecoration;
  BoxPainter _boxPainter;
  const double padding = 1;

  final DateTime viewStartDate = monthViewPainter.visibleDates[0];
  final DateTime viewEndDate =
      monthViewPainter.visibleDates[monthViewPainter.visibleDates.length - 1];
  final List<int> _selectedIndex =
      monthViewPainter._getSelectedIndexValues(viewStartDate, viewEndDate);
  for (int i = 0; i < monthViewPainter.visibleDates.length; i++) {
    int _currentIndex = i;
    if (monthViewPainter.isRtl) {
      final int rowIndex = i ~/ _IMonthViewPainter._numberOfDaysInWeek;
      _currentIndex = _getRtlIndex(_IMonthViewPainter._numberOfDaysInWeek,
              i % _IMonthViewPainter._numberOfDaysInWeek) +
          (rowIndex * _IMonthViewPainter._numberOfDaysInWeek);
    }

    _isCurrentDate = false;
    final DateTime _date = monthViewPainter.visibleDates[_currentIndex];
    _textStyle = _currentDatesTextStyle;
    _dateDecoration = monthViewPainter.cellStyle.cellDecoration;
    if (monthViewPainter.rowCount == 6) {
      if (_date.month == _nextMonth) {
        if (monthViewPainter.rowCount == 6 &&
            !monthViewPainter.showLeadingAndTailingDates) {
          _xPosition += cellWidth;
          if (_xPosition.round() >= size.width.round()) {
            _xPosition = 0;
            _yPosition += cellHeight;
          }
          continue;
        }

        _textStyle = _leadingDatesTextStyle;
        _dateDecoration = monthViewPainter.cellStyle.leadingDatesDecoration;
      } else if (_date.month == _previousMonth) {
        if (monthViewPainter.rowCount == 6 &&
            !monthViewPainter.showLeadingAndTailingDates) {
          _xPosition += cellWidth;
          if (_xPosition.round() >= size.width.round()) {
            _xPosition = 0;
            _yPosition += cellHeight;
          }
          continue;
        }

        _textStyle = _trailingDatesTextStyle;
        _dateDecoration = monthViewPainter.cellStyle.trailingDatesDecoration;
      }
    }

    if (_isWeekend(monthViewPainter.weekendDays, _date)) {
      _textStyle = monthViewPainter.cellStyle.weekendTextStyle ??
          (monthViewPainter.datePickerTheme != null &&
                  monthViewPainter.datePickerTheme.weekendDatesTextStyle != null
              ? monthViewPainter.datePickerTheme.weekendDatesTextStyle
              : _textStyle);
      _dateDecoration =
          monthViewPainter.cellStyle.weekendDatesDecoration ?? _dateDecoration;
    }

    if (_date.month == _today.month &&
        _date.day == _today.day &&
        _date.year == _today.year) {
      _textStyle = monthViewPainter.cellStyle.todayTextStyle ??
          monthViewPainter.datePickerTheme.todayTextStyle;
      _dateDecoration =
          monthViewPainter.cellStyle.todayCellDecoration ?? _dateDecoration;
      _isCurrentDate = true;
    }

    final bool _isEnableDate = _isEnabledDate(monthViewPainter.minDate,
        monthViewPainter.maxDate, monthViewPainter.enablePastDates, _date);
    if (!_isEnableDate) {
      _textStyle = _disabledDatesTextStyle;
      _dateDecoration = monthViewPainter.cellStyle.disabledDatesDecoration;
    }

    if (_isDateWithInVisibleDates(
        monthViewPainter.visibleDates, monthViewPainter.specialDates, _date)) {
      _textStyle = _specialDatesTextStyle;
      _dateDecoration = monthViewPainter.cellStyle.specialDatesDecoration;
    }

    final bool _isBlackedDate = _isDateWithInVisibleDates(
        monthViewPainter.visibleDates, monthViewPainter.blackoutDates, _date);
    if (_isBlackedDate) {
      _textStyle = monthViewPainter.cellStyle.blackoutDateTextStyle ??
          (monthViewPainter.datePickerTheme.blackoutDatesTextStyle != null
              ? monthViewPainter.datePickerTheme.blackoutDatesTextStyle
              : _textStyle.copyWith(decoration: TextDecoration.lineThrough));
      _dateDecoration = monthViewPainter.cellStyle.blackoutDatesDecoration;
    }

    if (_selectedIndex.contains(_currentIndex) &&
        !_isBlackedDate &&
        _isEnableDate) {
      monthViewPainter._selectionPainter =
          monthViewPainter._selectionPainter ?? Paint();
      monthViewPainter._selectionPainter.color =
          monthViewPainter.cellStyle.selectionColor ??
              monthViewPainter.datePickerTheme.selectionColor;
      //// Unwanted space shown at end of the rectangle while enable anti aliasing property.
      monthViewPainter._selectionPainter.isAntiAlias = false;
      monthViewPainter._selectionPainter.strokeWidth = 0.0;
      monthViewPainter._selectionPainter.style = PaintingStyle.fill;
      _textStyle = monthViewPainter._drawSelection(
          canvas,
          _xPosition,
          _yPosition,
          _currentIndex,
          _selectionTextStyle,
          _selectedRangeTextStyle);
    } else if (_dateDecoration != null) {
      _boxPainter = _dateDecoration.createBoxPainter();
      _boxPainter.paint(
          canvas,
          Offset(_xPosition + padding, _yPosition + padding),
          ImageConfiguration(
              size:
                  Size(cellWidth - (2 * padding), cellHeight - (2 * padding))));
    } else if (_isCurrentDate) {
      monthViewPainter._selectionPainter =
          monthViewPainter._selectionPainter ?? Paint();
      monthViewPainter._selectionPainter.color =
          monthViewPainter.todayHighlightColor ??
              monthViewPainter.datePickerTheme.todayHighlightColor;
      monthViewPainter._selectionPainter.isAntiAlias = true;
      monthViewPainter._selectionPainter.strokeWidth = 1.0;
      monthViewPainter._selectionPainter.style = PaintingStyle.stroke;

      if (monthViewPainter.selectionShape ==
          DateRangePickerSelectionShape.circle) {
        final double _centerXPosition = cellWidth / 2;
        final double _centerYPosition = cellHeight / 2;
        final double _radius = _getCellRadius(monthViewPainter.selectionRadius,
            _centerXPosition, _centerYPosition);
        canvas.drawCircle(
            Offset(
                _xPosition + _centerXPosition, _yPosition + _centerYPosition),
            _radius,
            monthViewPainter._selectionPainter);
      } else {
        canvas.drawRRect(
            RRect.fromRectAndRadius(
                Rect.fromLTRB(
                    _xPosition + padding,
                    _yPosition + padding,
                    _xPosition + cellWidth - padding,
                    _yPosition + cellHeight - padding),
                Radius.circular(cellHeight / 4 > 10 ? 10 : cellHeight / 4)),
            monthViewPainter._selectionPainter);
      }
    }

    final TextSpan _dateText = TextSpan(
      text: _date.day.toString(),
      style: _textStyle,
    );

    monthViewPainter._textPainter.text = _dateText;
    monthViewPainter._textPainter.layout(minWidth: 0, maxWidth: cellWidth);
    monthViewPainter._textPainter.paint(
        canvas,
        Offset(
            _xPosition,
            _yPosition +
                ((cellHeight - monthViewPainter._textPainter.height) / 2)));
    if (monthViewPainter._mouseHoverPosition != null) {
      if (_selectedIndex.contains(_currentIndex) ||
          _isBlackedDate ||
          !_isEnableDate) {
        _xPosition += cellWidth;
        if (_xPosition.round() >= size.width.round()) {
          _xPosition = 0;
          _yPosition += cellHeight;
        }
        continue;
      }

      if (_xPosition <= monthViewPainter._mouseHoverPosition.dx &&
          _xPosition + cellWidth >= monthViewPainter._mouseHoverPosition.dx &&
          _yPosition - 5 <= monthViewPainter._mouseHoverPosition.dy &&
          (_yPosition + cellHeight) - 5 >=
              monthViewPainter._mouseHoverPosition.dy) {
        monthViewPainter._selectionPainter =
            monthViewPainter._selectionPainter ?? Paint();
        monthViewPainter._selectionPainter.style = PaintingStyle.fill;
        monthViewPainter._selectionPainter.strokeWidth = 2;
        monthViewPainter._selectionPainter.color =
            monthViewPainter.cellStyle.selectionColor != null
                ? monthViewPainter.cellStyle.selectionColor.withOpacity(0.4)
                : monthViewPainter.datePickerTheme.selectionColor
                    .withOpacity(0.4);
        if (monthViewPainter.selectionShape ==
            DateRangePickerSelectionShape.circle) {
          final double _centerXPosition = cellWidth / 2;
          final double _centerYPosition = cellHeight / 2;
          final double _radius = _getCellRadius(
              monthViewPainter.selectionRadius,
              _centerXPosition,
              _centerYPosition);
          canvas.drawCircle(
              Offset(
                  _xPosition + _centerXPosition, _yPosition + _centerYPosition),
              _radius,
              monthViewPainter._selectionPainter);
        } else {
          canvas.drawRRect(
              RRect.fromRectAndRadius(
                  Rect.fromLTWH(_xPosition + 1, _yPosition + 1, cellWidth - 1,
                      cellHeight - 1),
                  Radius.circular(cellHeight / 4 > 10 ? 10 : cellHeight / 4)),
              monthViewPainter._selectionPainter);
        }
      }
    }

    _xPosition += cellWidth;
    if (_xPosition.round() >= size.width.round()) {
      _xPosition = 0;
      _yPosition += cellHeight;
    }
  }

  _yPosition = cellHeight;
}

List<CustomPainterSemantics> _getSemanticsBuilderForMonthView(
    Size size, int _rowCount, _IMonthViewPainter monthViewPainter) {
  final List<CustomPainterSemantics> _semanticsBuilder =
      <CustomPainterSemantics>[];
  double _left, _top, _cellWidth, _cellHeight;
  Map<String, double> _leftAndTopValue;
  _top = 0;
  _cellWidth = size.width / 7;
  _cellHeight = size.height / _rowCount;
  _left = monthViewPainter.isRtl ? size.width - _cellWidth : 0;
  for (int i = 0; i < monthViewPainter.visibleDates.length; i++) {
    if (!_isDateAsCurrentMonthDate(
        monthViewPainter
            .visibleDates[monthViewPainter.visibleDates.length ~/ 2],
        monthViewPainter.rowCount,
        monthViewPainter.showLeadingAndTailingDates,
        monthViewPainter.visibleDates[i])) {
      _leftAndTopValue = _getTopAndLeftValues(monthViewPainter.isRtl, _left,
          _top, _cellWidth, _cellHeight, size.width);
      _left = _leftAndTopValue['left'];
      _top = _leftAndTopValue['top'];
      continue;
    } else if (_isDateWithInVisibleDates(monthViewPainter.visibleDates,
        monthViewPainter.blackoutDates, monthViewPainter.visibleDates[i])) {
      _semanticsBuilder.add(CustomPainterSemantics(
        rect: Rect.fromLTWH(_left, _top, _cellWidth, _cellHeight),
        properties: SemanticsProperties(
          label: DateFormat('EEE, dd/MMMM/yyyy')
                  .format(monthViewPainter.visibleDates[i])
                  .toString() +
              ', Blackout date',
          textDirection: TextDirection.ltr,
        ),
      ));
      _leftAndTopValue = _getTopAndLeftValues(monthViewPainter.isRtl, _left,
          _top, _cellWidth, _cellHeight, size.width);
      _left = _leftAndTopValue['left'];
      _top = _leftAndTopValue['top'];
      continue;
    } else if (!_isEnabledDate(
        monthViewPainter.minDate,
        monthViewPainter.maxDate,
        monthViewPainter.enablePastDates,
        monthViewPainter.visibleDates[i])) {
      _semanticsBuilder.add(CustomPainterSemantics(
        rect: Rect.fromLTWH(_left, _top, _cellWidth, _cellHeight),
        properties: SemanticsProperties(
          label: DateFormat('EEE, dd/MMMM/yyyy')
                  .format(monthViewPainter.visibleDates[i])
                  .toString() +
              ', Disabled date',
          textDirection: TextDirection.ltr,
        ),
      ));
      _leftAndTopValue = _getTopAndLeftValues(monthViewPainter.isRtl, _left,
          _top, _cellWidth, _cellHeight, size.width);
      _left = _leftAndTopValue['left'];
      _top = _leftAndTopValue['top'];
      continue;
    }
    _semanticsBuilder.add(CustomPainterSemantics(
      rect: Rect.fromLTWH(_left, _top, _cellWidth, _cellHeight),
      properties: SemanticsProperties(
        label: DateFormat('EEE, dd/MMMM/yyyy')
            .format(monthViewPainter.visibleDates[i])
            .toString(),
        textDirection: TextDirection.ltr,
      ),
    ));
    _leftAndTopValue = _getTopAndLeftValues(monthViewPainter.isRtl, _left, _top,
        _cellWidth, _cellHeight, size.width);
    _left = _leftAndTopValue['left'];
    _top = _leftAndTopValue['top'];
  }

  return _semanticsBuilder;
}
