part of datepicker;

class _MonthViewMultiRangeSelectionPainter extends _IMonthViewPainter {
  _MonthViewMultiRangeSelectionPainter(
      this.visibleDates,
      this.rowCount,
      this.cellStyle,
      this.datePickerTheme,
      this.isRtl,
      this.todayHighlightColor,
      this.minDate,
      this.maxDate,
      this.enablePastDates,
      this.showLeadingAndTailingDates,
      this.blackoutDates,
      this.specialDates,
      this.weekendDays,
      this.selectedRanges,
      this.selectionShape,
      this.selectionRadius,
      this._mouseHoverPosition,
      this.selectionNotifier)
      : super(selectionNotifier: selectionNotifier);

  final int rowCount;
  final DateRangePickerMonthCellStyle cellStyle;
  final List<DateTime> visibleDates;
  final SfDateRangePickerThemeData datePickerTheme;
  final bool isRtl;
  final Color todayHighlightColor;
  final DateTime minDate;
  final DateTime maxDate;
  final bool enablePastDates;
  final bool showLeadingAndTailingDates;
  final List<DateTime> blackoutDates;
  final List<DateTime> specialDates;
  final List<int> weekendDays;
  final DateRangePickerSelectionShape selectionShape;
  final double selectionRadius;
  final Offset _mouseHoverPosition;
  List<PickerDateRange> selectedRanges;
  ValueNotifier<bool> selectionNotifier;
  Paint _selectionPainter;
  TextPainter _textPainter;
  double _cellWidth, _cellHeight;
  double _centerXPosition, _centerYPosition;
  List<List<int>> _selectedRangesIndex;

  @override
  TextStyle _drawSelection(Canvas canvas, double x, double y, int index,
      TextStyle selectionTextStyle, TextStyle selectionRangeTextStyle) {
    bool _isSelectedDate = false;
    bool _isStartRange = false;
    bool _isEndRange = false;
    bool _isBetweenRange = false;
    for (int j = 0; j < _selectedRangesIndex.length; j++) {
      final List<int> _rangeIndex = _selectedRangesIndex[j];
      if (!_rangeIndex.contains(index)) {
        continue;
      }

      if (_rangeIndex.length == 1) {
        _isSelectedDate = true;
      } else if (_rangeIndex[0] == index) {
        if (isRtl) {
          _isEndRange = true;
        } else {
          _isStartRange = true;
        }
      } else if (_rangeIndex[_rangeIndex.length - 1] == index) {
        if (isRtl) {
          _isStartRange = true;
        } else {
          _isEndRange = true;
        }
      } else {
        _isBetweenRange = true;
      }

      break;
    }

    final double _radius =
        _getCellRadius(selectionRadius, _centerXPosition, _centerYPosition);
    if (_isSelectedDate) {
      _selectionPainter.isAntiAlias = true;
      _selectionPainter.color = cellStyle.startRangeSelectionColor ??
          datePickerTheme.startRangeSelectionColor;
      if (selectionShape == DateRangePickerSelectionShape.circle) {
        _drawCircleSelection(canvas, x + _centerXPosition, y + _centerYPosition,
            _radius, _selectionPainter);
      } else {
        _drawFillSelection(
            canvas, x, y, _cellWidth, _cellHeight, _selectionPainter);
      }
    } else if (_isStartRange) {
      if (selectionShape == DateRangePickerSelectionShape.circle) {
        _selectionPainter.color = cellStyle.rangeSelectionColor ??
            datePickerTheme.rangeSelectionColor;
        final double _heightDifference = _cellHeight / 2 - _radius;
        final Rect _rect = Rect.fromLTRB(
            x + _centerXPosition,
            y + _heightDifference,
            x + _cellWidth,
            y + _cellHeight - _heightDifference);
        _drawStartEndRangeCircleSelection(
            canvas,
            x + _centerXPosition,
            y + _centerYPosition,
            _radius,
            _rect,
            _selectionPainter,
            cellStyle.startRangeSelectionColor ??
                datePickerTheme.startRangeSelectionColor);
      } else {
        _selectionPainter.color = cellStyle.startRangeSelectionColor ??
            datePickerTheme.startRangeSelectionColor;
        _drawStartRangeFillSelection(
            canvas, x, y, _cellWidth, _cellHeight, _selectionPainter);
      }
    } else if (_isEndRange) {
      if (selectionShape == DateRangePickerSelectionShape.circle) {
        _selectionPainter.color = cellStyle.rangeSelectionColor ??
            datePickerTheme.rangeSelectionColor;
        final double _heightDifference = _cellHeight / 2 - _radius;
        final Rect _rect = Rect.fromLTRB(x, y + _heightDifference,
            x + _centerXPosition, y + _cellHeight - _heightDifference);
        _drawStartEndRangeCircleSelection(
            canvas,
            x + _centerXPosition,
            y + _centerYPosition,
            _radius,
            _rect,
            _selectionPainter,
            cellStyle.endRangeSelectionColor ??
                datePickerTheme.endRangeSelectionColor);
      } else {
        _selectionPainter.color = cellStyle.endRangeSelectionColor ??
            datePickerTheme.endRangeSelectionColor;
        _drawEndRangeFillSelection(
            canvas, x, y, _cellWidth, _cellHeight, _selectionPainter);
      }
    } else if (_isBetweenRange) {
      double _heightDifference = _cellHeight / 2 - _radius;
      if (selectionShape == DateRangePickerSelectionShape.rectangle) {
        _heightDifference = 1;
      }

      _selectionPainter.color =
          cellStyle.rangeSelectionColor ?? datePickerTheme.rangeSelectionColor;
      _drawRectRangeSelection(canvas, x, y + _heightDifference, x + _cellWidth,
          y + _cellHeight - _heightDifference, _selectionPainter);
      return selectionRangeTextStyle;
    }

    return selectionTextStyle;
  }

  @override
  List<int> _getSelectedIndexValues(
      DateTime viewStartDate, DateTime viewEndDate) {
    final List<int> _selectedIndex = <int>[];
    _selectedRangesIndex = <List<int>>[];
    if (selectedRanges != null) {
      for (int j = 0; j < selectedRanges.length; j++) {
        final PickerDateRange _range = selectedRanges[j];
        final DateTime _startDate = _range.startDate;
        final DateTime _endDate = _range.endDate ?? _range.startDate;
        final List<int> _rangeIndex =
            _getSelectedRangeIndex(_startDate, _endDate, visibleDates);
        for (int i = 0; i < _rangeIndex.length; i++) {
          _selectedIndex.add(_rangeIndex[i]);
        }

        _selectedRangesIndex.add(_rangeIndex);
      }
    }

    return _selectedIndex;
  }

  @override
  void paint(Canvas canvas, Size size) {
    _cellWidth = size.width / _IMonthViewPainter._numberOfDaysInWeek;
    _cellHeight = size.height / rowCount;
    _centerXPosition = _cellWidth / 2;
    _centerYPosition = _cellHeight / 2;
    _drawMonthCellsAndSelection(canvas, size, this, _cellWidth, _cellHeight);
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    final _MonthViewMultiRangeSelectionPainter oldWidget = oldDelegate;
    return oldWidget.visibleDates != visibleDates ||
        oldWidget.rowCount != rowCount ||
        oldWidget.todayHighlightColor != todayHighlightColor ||
        oldWidget.enablePastDates != enablePastDates ||
        oldWidget.showLeadingAndTailingDates != showLeadingAndTailingDates ||
        oldWidget.cellStyle != cellStyle ||
        oldWidget.minDate != minDate ||
        oldWidget.maxDate != maxDate ||
        oldWidget.selectedRanges != selectedRanges ||
        oldWidget.blackoutDates != blackoutDates ||
        oldWidget.specialDates != specialDates ||
        oldWidget.selectionShape != selectionShape ||
        oldWidget.selectionRadius != selectionRadius ||
        oldWidget.isRtl != isRtl ||
        oldWidget.datePickerTheme != datePickerTheme ||
        (kIsWeb && oldWidget._mouseHoverPosition != _mouseHoverPosition);
  }

  @override
  void _updateSelection(_PickerStateArgs details) {
    if (_isDateRangesEquals(details._selectedRanges, selectedRanges)) {
      return;
    }

    selectedRanges = _cloneList(details._selectedRanges);
    selectionNotifier.value = !selectionNotifier.value;
  }

  @override
  SemanticsBuilderCallback get semanticsBuilder {
    return (Size size) {
      return _getSemanticsBuilderForMonthView(size, rowCount, this);
    };
  }

  @override
  bool shouldRebuildSemantics(CustomPainter oldDelegate) {
    final _MonthViewMultiRangeSelectionPainter _oldWidget = oldDelegate;
    return _oldWidget.visibleDates != visibleDates;
  }
}
