part of datepicker;

class _MonthViewRangeSelectionPainter extends _IMonthViewPainter {
  _MonthViewRangeSelectionPainter(
      this.visibleDates,
      this.rowCount,
      this.cellStyle,
      this.datePickerTheme,
      this.isRtl,
      this.todayHighlightColor,
      this.minDate,
      this.maxDate,
      this.enablePastDates,
      this.showLeadingAndTailingDates,
      this.blackoutDates,
      this.specialDates,
      this.weekendDays,
      this.selectedRange,
      this.selectionShape,
      this.selectionRadius,
      this._mouseHoverPosition,
      this.selectionNotifier)
      : super(selectionNotifier: selectionNotifier);

  final int rowCount;
  final DateRangePickerMonthCellStyle cellStyle;
  final List<DateTime> visibleDates;
  final SfDateRangePickerThemeData datePickerTheme;
  final bool isRtl;
  final Color todayHighlightColor;
  final DateTime minDate;
  final DateTime maxDate;
  final bool enablePastDates;
  final bool showLeadingAndTailingDates;
  final List<DateTime> blackoutDates;
  final List<int> weekendDays;
  final List<DateTime> specialDates;
  final DateRangePickerSelectionShape selectionShape;
  final double selectionRadius;
  final Offset _mouseHoverPosition;
  PickerDateRange selectedRange;
  ValueNotifier<bool> selectionNotifier;
  Paint _selectionPainter;
  TextPainter _textPainter;
  double _cellWidth, _cellHeight;
  double _centerXPosition, _centerYPosition;
  List<int> _selectedIndex;

  @override
  TextStyle _drawSelection(Canvas canvas, double x, double y, int index,
      TextStyle selectionTextStyle, TextStyle selectionRangeTextStyle) {
    bool _isSelectedDate = false;
    bool _isStartRange = false;
    bool _isEndRange = false;
    bool _isBetweenRange = false;
    if (_selectedIndex.length == 1) {
      _isSelectedDate = true;
    } else if (_selectedIndex[0] == index) {
      if (isRtl) {
        _isEndRange = true;
      } else {
        _isStartRange = true;
      }
    } else if (_selectedIndex[_selectedIndex.length - 1] == index) {
      if (isRtl) {
        _isStartRange = true;
      } else {
        _isEndRange = true;
      }
    } else {
      _isBetweenRange = true;
    }

    final double _radius =
        _getCellRadius(selectionRadius, _centerXPosition, _centerYPosition);
    if (_isSelectedDate) {
      _selectionPainter.isAntiAlias = true;
      _selectionPainter.color = cellStyle.startRangeSelectionColor ??
          datePickerTheme.startRangeSelectionColor;
      if (selectionShape == DateRangePickerSelectionShape.circle) {
        _drawCircleSelection(canvas, x + _centerXPosition, y + _centerYPosition,
            _radius, _selectionPainter);
      } else {
        _drawFillSelection(
            canvas, x, y, _cellWidth, _cellHeight, _selectionPainter);
      }
    } else if (_isStartRange) {
      if (selectionShape == DateRangePickerSelectionShape.circle) {
        final double _heightDifference = _cellHeight / 2 - _radius;
        _selectionPainter.color = cellStyle.rangeSelectionColor ??
            datePickerTheme.rangeSelectionColor;
        final Rect _rect = Rect.fromLTRB(
            x + _centerXPosition,
            y + _heightDifference,
            x + _cellWidth,
            y + _cellHeight - _heightDifference);
        _drawStartEndRangeCircleSelection(
            canvas,
            x + _centerXPosition,
            y + _centerYPosition,
            _radius,
            _rect,
            _selectionPainter,
            cellStyle.startRangeSelectionColor ??
                datePickerTheme.startRangeSelectionColor);
      } else {
        _selectionPainter.color = cellStyle.startRangeSelectionColor ??
            datePickerTheme.startRangeSelectionColor;
        _drawStartRangeFillSelection(
            canvas, x, y, _cellWidth, _cellHeight, _selectionPainter);
      }
    } else if (_isEndRange) {
      if (selectionShape == DateRangePickerSelectionShape.circle) {
        final double _heightDifference = _cellHeight / 2 - _radius;
        _selectionPainter.color = cellStyle.rangeSelectionColor ??
            datePickerTheme.rangeSelectionColor;
        final Rect _rect = Rect.fromLTRB(x, y + _heightDifference,
            x + _centerXPosition, y + _cellHeight - _heightDifference);
        _drawStartEndRangeCircleSelection(
            canvas,
            x + _centerXPosition,
            y + _centerYPosition,
            _radius,
            _rect,
            _selectionPainter,
            cellStyle.endRangeSelectionColor ??
                datePickerTheme.endRangeSelectionColor);
      } else {
        _selectionPainter.color = cellStyle.endRangeSelectionColor ??
            datePickerTheme.endRangeSelectionColor;
        _drawEndRangeFillSelection(
            canvas, x, y, _cellWidth, _cellHeight, _selectionPainter);
      }
    } else if (_isBetweenRange) {
      double _heightDifference = _cellHeight / 2 - _radius;
      if (selectionShape == DateRangePickerSelectionShape.rectangle) {
        _heightDifference = 1;
      }

      _selectionPainter.color =
          cellStyle.rangeSelectionColor ?? datePickerTheme.rangeSelectionColor;
      _drawRectRangeSelection(canvas, x, y + _heightDifference, x + _cellWidth,
          y + _cellHeight - _heightDifference, _selectionPainter);
      return selectionRangeTextStyle;
    }

    return selectionTextStyle;
  }

  @override
  List<int> _getSelectedIndexValues(
      DateTime viewStartDate, DateTime viewEndDate) {
    _selectedIndex = <int>[];
    if (selectedRange != null) {
      final DateTime _startDate = selectedRange.startDate;
      final DateTime _endDate =
          selectedRange.endDate ?? selectedRange.startDate;
      _selectedIndex =
          _getSelectedRangeIndex(_startDate, _endDate, visibleDates);
    }

    return _selectedIndex;
  }

  @override
  void paint(Canvas canvas, Size size) {
    _cellWidth = size.width / _IMonthViewPainter._numberOfDaysInWeek;
    _cellHeight = size.height / rowCount;
    _centerXPosition = _cellWidth / 2;
    _centerYPosition = _cellHeight / 2;
    _drawMonthCellsAndSelection(canvas, size, this, _cellWidth, _cellHeight);
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    final _MonthViewRangeSelectionPainter oldWidget = oldDelegate;
    return oldWidget.visibleDates != visibleDates ||
        oldWidget.rowCount != rowCount ||
        oldWidget.todayHighlightColor != todayHighlightColor ||
        oldWidget.enablePastDates != enablePastDates ||
        oldWidget.showLeadingAndTailingDates != showLeadingAndTailingDates ||
        oldWidget.cellStyle != cellStyle ||
        oldWidget.minDate != minDate ||
        oldWidget.maxDate != maxDate ||
        oldWidget.selectedRange != selectedRange ||
        oldWidget.blackoutDates != blackoutDates ||
        oldWidget.specialDates != specialDates ||
        oldWidget.selectionShape != selectionShape ||
        oldWidget.selectionRadius != selectionRadius ||
        oldWidget.isRtl != isRtl ||
        oldWidget.datePickerTheme != datePickerTheme ||
        (kIsWeb && oldWidget._mouseHoverPosition != _mouseHoverPosition);
  }

  @override
  void _updateSelection(_PickerStateArgs details) {
    if (_isRangeEquals(details._selectedRange, selectedRange)) {
      return;
    }

    selectedRange = details._selectedRange;
    selectionNotifier.value = !selectionNotifier.value;
  }

  @override
  SemanticsBuilderCallback get semanticsBuilder {
    return (Size size) {
      return _getSemanticsBuilderForMonthView(size, rowCount, this);
    };
  }

  @override
  bool shouldRebuildSemantics(CustomPainter oldDelegate) {
    final _MonthViewRangeSelectionPainter _oldWidget = oldDelegate;
    return _oldWidget.visibleDates != visibleDates;
  }
}
