part of datepicker;

class _PickerView extends StatefulWidget {
  const _PickerView(this.picker, this.controller, this.visibleDates, this.width,
      this.height, this._datePickerTheme, this._focusNode,
      {Key key,
      this.getPickerStateDetails,
      this.updatePickerStateDetails,
      this.isRtl})
      : super(key: key);

  final List<DateTime> visibleDates;
  final SfDateRangePicker picker;
  final DateRangePickerController controller;
  final double width;
  final double height;
  final _UpdatePickerState getPickerStateDetails;
  final _UpdatePickerState updatePickerStateDetails;
  final SfDateRangePickerThemeData _datePickerTheme;
  final bool isRtl;
  final FocusNode _focusNode;

  @override
  _PickerViewState createState() => _PickerViewState();
}

class _PickerViewState extends State<_PickerView>
    with TickerProviderStateMixin {
  int _numberOfDaysInWeek;
  _PickerStateArgs _pickerStateDetails;
  _IMonthViewPainter _monthView;
  Offset _mouseHoverPosition;

  //// drag start boolean variable used to identify whether the drag started or not
  //// For example., if user start drag from disabled date then the start date of the range not created
  //// so in drag update method update the end date of existing selected range.
  bool _isDragStart;

  @override
  void initState() {
    _pickerStateDetails = _PickerStateArgs();
    _numberOfDaysInWeek = 7;
    _isDragStart = false;
    super.initState();
  }

  // Returns the month view  as a child for the calendar view.
  Widget _addMonthView(
      Locale _locale, SfDateRangePickerThemeData _datePickerTheme) {
    double viewHeaderHeight = widget.picker.monthViewSettings.viewHeaderHeight;
    if (widget.controller.view == DateRangePickerView.month &&
        widget.picker.navigationDirection ==
            DateRangePickerNavigationDirection.vertical) {
      viewHeaderHeight = 0;
    }

    final double height = widget.height - viewHeaderHeight;
    _monthView = _getMonthView(_locale, widget._datePickerTheme);
    return Stack(
      children: <Widget>[
        _getViewHeader(viewHeaderHeight, _locale, _datePickerTheme),
        Positioned(
          left: 0,
          top: viewHeaderHeight,
          right: 0,
          height: height,
          child: RepaintBoundary(
            child: CustomPaint(
              painter: _monthView,
              size: Size(widget.width, height),
            ),
          ),
        ),
      ],
    );
  }

  _IMonthViewPainter _getMonthView(
      Locale _locale, SfDateRangePickerThemeData _datePickerTheme) {
    if (widget.picker.selectionMode == DateRangePickerSelectionMode.single) {
      return _MonthViewSingleSelectionPainter(
          widget.visibleDates,
          widget.picker.monthViewSettings.numberOfWeeksInView,
          widget.picker.monthCellStyle,
          widget._datePickerTheme,
          widget.isRtl,
          widget.picker.todayHighlightColor,
          widget.picker.minDate,
          widget.picker.maxDate,
          widget.picker.enablePastDates,
          widget.picker.monthViewSettings.showTrailingAndLeadingDates,
          widget.picker.monthViewSettings.blackoutDates,
          widget.picker.monthViewSettings.specialDates,
          widget.picker.monthViewSettings.weekendDays,
          _pickerStateDetails._selectedDate,
          widget.picker.selectionShape,
          widget.picker.monthViewSettings.selectionRadius,
          _mouseHoverPosition,
          ValueNotifier<bool>(false));
    } else if (widget.picker.selectionMode ==
        DateRangePickerSelectionMode.multiple) {
      return _MonthViewMultiSelectionPainter(
          widget.visibleDates,
          widget.picker.monthViewSettings.numberOfWeeksInView,
          widget.picker.monthCellStyle,
          widget._datePickerTheme,
          widget.isRtl,
          widget.picker.todayHighlightColor,
          widget.picker.minDate,
          widget.picker.maxDate,
          widget.picker.enablePastDates,
          widget.picker.monthViewSettings.showTrailingAndLeadingDates,
          widget.picker.monthViewSettings.blackoutDates,
          widget.picker.monthViewSettings.specialDates,
          widget.picker.monthViewSettings.weekendDays,
          _cloneList(_pickerStateDetails._selectedDates),
          widget.picker.selectionShape,
          widget.picker.monthViewSettings.selectionRadius,
          _mouseHoverPosition,
          ValueNotifier<bool>(false));
    } else if (widget.picker.selectionMode ==
        DateRangePickerSelectionMode.range) {
      return _MonthViewRangeSelectionPainter(
          widget.visibleDates,
          widget.picker.monthViewSettings.numberOfWeeksInView,
          widget.picker.monthCellStyle,
          widget._datePickerTheme,
          widget.isRtl,
          widget.picker.todayHighlightColor,
          widget.picker.minDate,
          widget.picker.maxDate,
          widget.picker.enablePastDates,
          widget.picker.monthViewSettings.showTrailingAndLeadingDates,
          widget.picker.monthViewSettings.blackoutDates,
          widget.picker.monthViewSettings.specialDates,
          widget.picker.monthViewSettings.weekendDays,
          _pickerStateDetails._selectedRange,
          widget.picker.selectionShape,
          widget.picker.monthViewSettings.selectionRadius,
          _mouseHoverPosition,
          ValueNotifier<bool>(false));
    } else if (widget.picker.selectionMode ==
        DateRangePickerSelectionMode.multiRange) {
      return _MonthViewMultiRangeSelectionPainter(
          widget.visibleDates,
          widget.picker.monthViewSettings.numberOfWeeksInView,
          widget.picker.monthCellStyle,
          widget._datePickerTheme,
          widget.isRtl,
          widget.picker.todayHighlightColor,
          widget.picker.minDate,
          widget.picker.maxDate,
          widget.picker.enablePastDates,
          widget.picker.monthViewSettings.showTrailingAndLeadingDates,
          widget.picker.monthViewSettings.blackoutDates,
          widget.picker.monthViewSettings.specialDates,
          widget.picker.monthViewSettings.weekendDays,
          _cloneList(_pickerStateDetails._selectedRanges),
          widget.picker.selectionShape,
          widget.picker.monthViewSettings.selectionRadius,
          _mouseHoverPosition,
          ValueNotifier<bool>(false));
    }

    return null;
  }

  Widget _getViewHeader(double viewHeaderHeight, Locale _locale,
      SfDateRangePickerThemeData _datePickerTheme) {
    if (viewHeaderHeight == 0) {
      return Positioned(
          left: 0,
          top: 0,
          right: 0,
          height: viewHeaderHeight,
          child: Container());
    }

    Color todayTextColor =
        widget.picker.monthCellStyle.todayTextStyle != null &&
                widget.picker.monthCellStyle.todayTextStyle.color != null
            ? widget.picker.monthCellStyle.todayTextStyle.color
            : (widget.picker.todayHighlightColor != null &&
                    widget.picker.todayHighlightColor != Colors.transparent
                ? widget.picker.todayHighlightColor
                : widget.picker.todayHighlightColor ??
                    widget._datePickerTheme.todayHighlightColor);
    if (todayTextColor == Colors.transparent) {
      todayTextColor = Colors.blue;
    }
    return Positioned(
      left: 0,
      top: 0,
      right: 0,
      height: viewHeaderHeight,
      child: Container(
        color:
            widget.picker.monthViewSettings.viewHeaderStyle.backgroundColor ??
                widget._datePickerTheme.viewHeaderBackgroundColor,
        child: RepaintBoundary(
          child: CustomPaint(
            painter: _PickerViewHeaderPainter(
                widget.visibleDates,
                widget.picker.monthViewSettings.viewHeaderStyle,
                viewHeaderHeight,
                widget.picker.monthViewSettings,
                widget._datePickerTheme,
                _locale,
                widget.isRtl,
                widget.picker.monthCellStyle,
                todayTextColor),
          ),
        ),
      ),
    );
  }

  void _updateTapCallback(TapUpDetails details) {
    if (widget.controller.view == DateRangePickerView.month) {
      double viewHeaderHeight =
          widget.picker.monthViewSettings.viewHeaderHeight;
      if (widget.picker.navigationDirection ==
          DateRangePickerNavigationDirection.vertical) {
        viewHeaderHeight = 0;
      }

      if (details.localPosition.dy < viewHeaderHeight) {
        return;
      }

      if (details.localPosition.dy > viewHeaderHeight) {
        _handleTouch(
            Offset(details.localPosition.dx,
                details.localPosition.dy - viewHeaderHeight),
            details);
      }
    } else {
      _handleYearPanelSelection(
          Offset(details.localPosition.dx, details.localPosition.dy));
    }

    if (!widget._focusNode.hasFocus) {
      widget._focusNode.requestFocus();
    }
  }

  @override
  Widget build(BuildContext context) {
    final Locale _locale = Localizations.localeOf(context);
    _pickerStateDetails._currentDate = null;
    _resetPickerStateDetails();
    widget.getPickerStateDetails(_pickerStateDetails);

    if (widget.controller.view == DateRangePickerView.month) {
      return GestureDetector(
        child: MouseRegion(
          onEnter: _pointerEnterEvent,
          onHover: _pointerHoverEvent,
          onExit: _pointerExitEvent,
          child: _addMonthView(_locale, widget._datePickerTheme),
        ),
        onTapUp: _updateTapCallback,
        onHorizontalDragStart: _getDragStartCallback(),
        onVerticalDragStart: _getDragStartCallback(),
        onHorizontalDragUpdate: _getDragUpdateCallback(),
        onVerticalDragUpdate: _getDragUpdateCallback(),
      );
    } else {
      return GestureDetector(
        child: MouseRegion(
          onEnter: _pointerEnterEvent,
          onHover: _pointerHoverEvent,
          onExit: _pointerExitEvent,
          child: _addYearView(_locale),
        ),
        onTapUp: _updateTapCallback,
      );
    }
  }

  void _updateMouseHover(dynamic event) {
    final RenderBox box = context.findRenderObject();
    final dynamic localPosition = box.globalToLocal(event.position);
    final double _viewHeaderHeight =
        widget.controller.view == DateRangePickerView.month
            ? widget.picker.monthViewSettings.viewHeaderHeight
            : 0;
    final double _xPosition = localPosition.dx;
    final double _yPosition = localPosition.dy - _viewHeaderHeight;

    if (localPosition.dy < _viewHeaderHeight) {
      return;
    }

    setState(() {
      _mouseHoverPosition = Offset(_xPosition, _yPosition);
    });
  }

  dynamic _pointerEnterEvent(PointerEnterEvent event) {
    _updateMouseHover(event);
  }

  dynamic _pointerHoverEvent(PointerHoverEvent event) {
    _updateMouseHover(event);
  }

  dynamic _pointerExitEvent(PointerExitEvent event) {
    setState(() {
      _mouseHoverPosition = null;
    });
  }

  Widget _addYearView(Locale _locale) {
    return RepaintBoundary(
      child: CustomPaint(
        painter: _getYearView(_locale),
        size: Size(widget.width, widget.height),
      ),
    );
  }

  _IPickerYearView _getYearView(Locale _locale) {
    if (widget.controller.view == DateRangePickerView.year) {
      return _PickerYearViewPainter(
          widget.visibleDates,
          widget.picker.yearCellStyle,
          widget.picker.minDate,
          widget.picker.maxDate,
          widget.picker.enablePastDates,
          widget.picker.todayHighlightColor,
          widget.picker.selectionShape,
          widget.picker.monthFormat,
          widget.isRtl,
          widget._datePickerTheme,
          _locale,
          _mouseHoverPosition,
          widget.picker.monthCellStyle.selectionColor);
    } else if (widget.controller.view == DateRangePickerView.decade) {
      return _PickerDecadeViewPainter(
          widget.visibleDates,
          widget.picker.yearCellStyle,
          widget.picker.minDate,
          widget.picker.maxDate,
          widget.picker.enablePastDates,
          widget.picker.todayHighlightColor,
          widget.picker.selectionShape,
          widget.isRtl,
          widget._datePickerTheme,
          _locale,
          _mouseHoverPosition,
          widget.picker.monthCellStyle.selectionColor);
    } else if (widget.controller.view == DateRangePickerView.century) {
      return _PickerCenturyViewPainter(
          widget.visibleDates,
          widget.picker.yearCellStyle,
          widget.picker.minDate,
          widget.picker.maxDate,
          widget.picker.enablePastDates,
          widget.picker.todayHighlightColor,
          widget.picker.selectionShape,
          widget.isRtl,
          widget._datePickerTheme,
          _locale,
          _mouseHoverPosition,
          widget.picker.monthCellStyle.selectionColor);
    }

    return null;
  }

  void _resetPickerStateDetails() {
    _pickerStateDetails._currentViewVisibleDates = null;
    _pickerStateDetails._selectedDate = null;
    _pickerStateDetails._selectedDates = null;
    _pickerStateDetails._selectedRange = null;
    _pickerStateDetails._selectedRanges = null;
    _pickerStateDetails._view = null;
  }

  dynamic _getDragStartCallback() {
    //// return drag start start event when selection mode as range or multi range with month view.
    if (widget.controller.view != DateRangePickerView.month ||
        !widget.picker.monthViewSettings.enableSwipeSelection) {
      return null;
    }

    if (widget.picker.selectionMode == DateRangePickerSelectionMode.range ||
        widget.picker.selectionMode ==
            DateRangePickerSelectionMode.multiRange) {
      return _dragStart;
    }

    return null;
  }

  dynamic _getDragUpdateCallback() {
    //// return drag update start event when selection mode as range or multi range with month view.
    if (widget.controller.view != DateRangePickerView.month ||
        !widget.picker.monthViewSettings.enableSwipeSelection) {
      return null;
    }

    if (widget.picker.selectionMode == DateRangePickerSelectionMode.range ||
        widget.picker.selectionMode ==
            DateRangePickerSelectionMode.multiRange) {
      return _dragUpdate;
    }

    return null;
  }

  int _getYearViewIndex(double xPosition, double yPosition) {
    int rowIndex, columnIndex;
    final double cellWidth = widget.width / _IPickerYearView._maxColumnCount;
    final double cellHeight = widget.height / _IPickerYearView._maxRowCount;
    int index = -1;
    if (yPosition < 0 || xPosition < 0) {
      return index;
    }

    rowIndex = xPosition ~/ cellWidth;
    if (rowIndex >= _IPickerYearView._maxColumnCount) {
      rowIndex = _IPickerYearView._maxColumnCount - 1;
    } else if (rowIndex < 0) {
      return index;
    }

    if (widget.isRtl) {
      rowIndex = _getRtlIndex(_IPickerYearView._maxColumnCount, rowIndex);
    }

    columnIndex = yPosition ~/ cellHeight;
    if (columnIndex >= _IPickerYearView._maxRowCount) {
      columnIndex = _IPickerYearView._maxRowCount - 1;
    } else if (columnIndex < 0) {
      return index;
    }

    index = (columnIndex * _IPickerYearView._maxColumnCount) + rowIndex;
    return index;
  }

  int _getSelectedIndex(double xPosition, double yPosition) {
    int rowIndex, columnIndex;
    double cellWidth = 0;
    double cellHeight = 0;
    int index = -1;
    final double _width = widget.width;
    if (yPosition < 0 || xPosition < 0) {
      return index;
    }

    double viewHeaderHeight = widget.picker.monthViewSettings.viewHeaderHeight;
    if (widget.controller.view == DateRangePickerView.month &&
        widget.picker.navigationDirection ==
            DateRangePickerNavigationDirection.vertical) {
      viewHeaderHeight = 0;
    }

    cellWidth = _width / _numberOfDaysInWeek;
    cellHeight = (widget.height - viewHeaderHeight) /
        widget.picker.monthViewSettings.numberOfWeeksInView;
    rowIndex = (xPosition / cellWidth).truncate();
    if (rowIndex >= _numberOfDaysInWeek) {
      rowIndex = _numberOfDaysInWeek - 1;
    } else if (rowIndex < 0) {
      return index;
    }

    if (widget.isRtl) {
      rowIndex = _getRtlIndex(_numberOfDaysInWeek, rowIndex);
    }

    columnIndex = (yPosition / cellHeight).truncate();
    if (columnIndex >= widget.picker.monthViewSettings.numberOfWeeksInView) {
      columnIndex = widget.picker.monthViewSettings.numberOfWeeksInView - 1;
    } else if (columnIndex < 0) {
      return index;
    }

    index = (columnIndex * _numberOfDaysInWeek) + rowIndex;
    return index;
  }

  void _dragStart(DragStartDetails details) {
    //// Set drag start value as false, identifies the start date of the range not updated.
    _isDragStart = false;
    _resetPickerStateDetails();
    widget.getPickerStateDetails(_pickerStateDetails);
    final double xPosition = details.localPosition.dx;
    double yPosition = details.localPosition.dy;
    if (widget.controller.view == DateRangePickerView.month &&
        widget.picker.navigationDirection ==
            DateRangePickerNavigationDirection.horizontal) {
      yPosition = details.localPosition.dy -
          widget.picker.monthViewSettings.viewHeaderHeight;
    }

    final int index = _getSelectedIndex(xPosition, yPosition);
    if (index == -1) {
      return;
    }

    final DateTime _selectedDate = widget.visibleDates[index];
    if (!_isEnabledDate(widget.picker.minDate, widget.picker.maxDate,
        widget.picker.enablePastDates, _selectedDate)) {
      return;
    }

    if (!_isDateAsCurrentMonthDate(
        widget.visibleDates[widget.visibleDates.length ~/ 2],
        widget.picker.monthViewSettings.numberOfWeeksInView,
        widget.picker.monthViewSettings.showTrailingAndLeadingDates,
        _selectedDate)) {
      return;
    }

    if (_isDateWithInVisibleDates(widget.visibleDates,
        widget.picker.monthViewSettings.blackoutDates, _selectedDate)) {
      return;
    }

    //// Set drag start value as false, identifies the start date of the range updated.
    _isDragStart = true;
    if (widget.picker.selectionMode == DateRangePickerSelectionMode.range) {
      _pickerStateDetails._selectedRange = PickerDateRange(_selectedDate, null);
      _monthView._updateSelection(_pickerStateDetails);
    } else if (widget.picker.selectionMode ==
        DateRangePickerSelectionMode.multiRange) {
      _pickerStateDetails._selectedRanges ??= <PickerDateRange>[];
      _pickerStateDetails._selectedRanges
          .add(PickerDateRange(_selectedDate, null));
      _removeInterceptRanges(
          _pickerStateDetails._selectedRanges,
          _pickerStateDetails
              ._selectedRanges[_pickerStateDetails._selectedRanges.length - 1]);
      _monthView._updateSelection(_pickerStateDetails);
    }

    widget.updatePickerStateDetails(_pickerStateDetails);
  }

  void _dragUpdate(DragUpdateDetails details) {
    _resetPickerStateDetails();
    widget.getPickerStateDetails(_pickerStateDetails);
    final double xPosition = details.localPosition.dx;
    double yPosition = details.localPosition.dy;
    if (widget.controller.view == DateRangePickerView.month &&
        widget.picker.navigationDirection ==
            DateRangePickerNavigationDirection.horizontal) {
      yPosition = details.localPosition.dy -
          widget.picker.monthViewSettings.viewHeaderHeight;
    }

    final int index = _getSelectedIndex(xPosition, yPosition);
    if (index == -1) {
      return;
    }

    final DateTime _selectedDate = widget.visibleDates[index];
    if (!_isEnabledDate(widget.picker.minDate, widget.picker.maxDate,
        widget.picker.enablePastDates, _selectedDate)) {
      return;
    }

    if (!_isDateAsCurrentMonthDate(
        widget.visibleDates[widget.visibleDates.length ~/ 2],
        widget.picker.monthViewSettings.numberOfWeeksInView,
        widget.picker.monthViewSettings.showTrailingAndLeadingDates,
        _selectedDate)) {
      return;
    }

    if (_isDateWithInVisibleDates(widget.visibleDates,
        widget.picker.monthViewSettings.blackoutDates, _selectedDate)) {
      return;
    }

    if (widget.picker.selectionMode == DateRangePickerSelectionMode.range) {
      //// Check the start date of the range updated or not, if not updated then create the new range.
      if (!_isDragStart) {
        _pickerStateDetails._selectedRange =
            PickerDateRange(_selectedDate, null);
      } else {
        if (_pickerStateDetails._selectedRange != null &&
            _pickerStateDetails._selectedRange.startDate != null) {
          _pickerStateDetails._selectedRange = PickerDateRange(
              _pickerStateDetails._selectedRange.startDate, _selectedDate);
        } else {
          _pickerStateDetails._selectedRange =
              PickerDateRange(_selectedDate, null);
        }
      }
      _monthView._updateSelection(_pickerStateDetails);
    } else if (widget.picker.selectionMode ==
        DateRangePickerSelectionMode.multiRange) {
      _pickerStateDetails._selectedRanges ??= <PickerDateRange>[];
      final int _count = _pickerStateDetails._selectedRanges.length;
      PickerDateRange _lastRange;
      if (_count > 0) {
        _lastRange = _pickerStateDetails._selectedRanges[_count - 1];
      }

      //// Check the start date of the range updated or not, if not updated then create the new range.
      if (!_isDragStart) {
        _pickerStateDetails._selectedRanges
            .add(PickerDateRange(_selectedDate, null));
      } else {
        if (_lastRange != null && _lastRange.startDate != null) {
          final PickerDateRange _newRange =
              PickerDateRange(_lastRange.startDate, _selectedDate);
          _pickerStateDetails._selectedRanges[_count - 1] = _newRange;
        } else {
          _pickerStateDetails._selectedRanges
              .add(PickerDateRange(_selectedDate, null));
        }
      }

      _removeInterceptRanges(
          _pickerStateDetails._selectedRanges,
          _pickerStateDetails
              ._selectedRanges[_pickerStateDetails._selectedRanges.length - 1]);
      _monthView._updateSelection(_pickerStateDetails);
    }

    //// Set drag start value as false, identifies the start date of the range updated.
    _isDragStart = true;
    widget.updatePickerStateDetails(_pickerStateDetails);
  }

  void _handleTouch(Offset details, TapUpDetails tapUpDetails) {
    _resetPickerStateDetails();
    widget.getPickerStateDetails(_pickerStateDetails);
    if (widget.controller.view == DateRangePickerView.month) {
      final int index = _getSelectedIndex(details.dx, details.dy);
      final DateTime _selectedDate = widget.visibleDates[index];
      if (!_isEnabledDate(widget.picker.minDate, widget.picker.maxDate,
          widget.picker.enablePastDates, _selectedDate)) {
        return;
      }

      if (!_isDateAsCurrentMonthDate(
          widget.visibleDates[widget.visibleDates.length ~/ 2],
          widget.picker.monthViewSettings.numberOfWeeksInView,
          widget.picker.monthViewSettings.showTrailingAndLeadingDates,
          _selectedDate)) {
        return;
      }

      if (_isDateWithInVisibleDates(widget.visibleDates,
          widget.picker.monthViewSettings.blackoutDates, _selectedDate)) {
        return;
      }

      _drawSelection(_selectedDate);
      widget.updatePickerStateDetails(_pickerStateDetails);
    }
  }

  void _handleYearPanelSelection(Offset details) {
    final int _selectedIndex = _getYearViewIndex(details.dx, details.dy);
    if (_selectedIndex == -1 || _selectedIndex >= 12) {
      return;
    }

    final DateTime _date = widget.visibleDates[_selectedIndex];
    _resetPickerStateDetails();
    widget.getPickerStateDetails(_pickerStateDetails);
    if (widget.controller.view == DateRangePickerView.century) {
      final int _year = _date.year ~/ 10;
      final int _minYear = widget.picker.minDate.year ~/ 10;
      final int _maxYear = widget.picker.maxDate.year ~/ 10;
      if (_year < _minYear || _year > _maxYear) {
        return;
      }

      _pickerStateDetails._view = DateRangePickerView.decade;
    } else if (widget.controller.view == DateRangePickerView.decade) {
      final int _year = _date.year;
      if (_year < widget.picker.minDate.year ||
          _year > widget.picker.maxDate.year) {
        return;
      }

      _pickerStateDetails._view = DateRangePickerView.year;
    } else if (widget.controller.view == DateRangePickerView.year) {
      final int _year = _date.year;
      final int _month = _date.month;
      final int _minYear = widget.picker.minDate.year;
      final int _maxYear = widget.picker.maxDate.year;
      if ((_year < _minYear ||
              (_year == _minYear && _month < widget.picker.minDate.month)) ||
          (_year > _maxYear ||
              (_year == _maxYear && _month > widget.picker.maxDate.month))) {
        return;
      }

      _pickerStateDetails._view = DateRangePickerView.month;
    }

    _pickerStateDetails._currentDate = _date;
    widget.updatePickerStateDetails(_pickerStateDetails);
  }

  void _drawSelection(DateTime _selectedDate) {
    if (widget.picker.selectionMode == DateRangePickerSelectionMode.single) {
      if (widget.picker.toggleDaySelection &&
          isSameDate(_selectedDate, _pickerStateDetails._selectedDate)) {
        _selectedDate = null;
      }

      _pickerStateDetails._selectedDate = _selectedDate;
      _monthView._updateSelection(_pickerStateDetails);
    } else if (widget.picker.selectionMode ==
        DateRangePickerSelectionMode.multiple) {
      final int selectedIndex = _isDateIndexInCollection(
          _pickerStateDetails._selectedDates, _selectedDate);
      if (selectedIndex == -1) {
        _pickerStateDetails._selectedDates ??= <DateTime>[];
        _pickerStateDetails._selectedDates.add(_selectedDate);
      } else {
        _pickerStateDetails._selectedDates.removeAt(selectedIndex);
      }

      _monthView._updateSelection(_pickerStateDetails);
    } else if (widget.picker.selectionMode ==
        DateRangePickerSelectionMode.range) {
      if (_pickerStateDetails._selectedRange != null &&
          _pickerStateDetails._selectedRange.startDate != null &&
          (_pickerStateDetails._selectedRange.endDate == null ||
              isSameDate(_pickerStateDetails._selectedRange.startDate,
                  _pickerStateDetails._selectedRange.endDate))) {
        _pickerStateDetails._selectedRange = PickerDateRange(
            _pickerStateDetails._selectedRange.startDate, _selectedDate);
      } else {
        _pickerStateDetails._selectedRange =
            PickerDateRange(_selectedDate, null);
      }

      _monthView._updateSelection(_pickerStateDetails);
    } else if (widget.picker.selectionMode ==
        DateRangePickerSelectionMode.multiRange) {
      _pickerStateDetails._selectedRanges ??= <PickerDateRange>[];
      int _count = _pickerStateDetails._selectedRanges.length;
      PickerDateRange _lastRange;
      if (_count > 0) {
        _lastRange = _pickerStateDetails._selectedRanges[_count - 1];
      }

      if (_lastRange != null &&
          _lastRange.startDate != null &&
          (_lastRange.endDate == null ||
              isSameDate(_lastRange.startDate, _lastRange.endDate))) {
        final PickerDateRange _newRange =
            PickerDateRange(_lastRange.startDate, _selectedDate);
        _pickerStateDetails._selectedRanges[_count - 1] = _newRange;
      } else {
        _pickerStateDetails._selectedRanges
            .add(PickerDateRange(_selectedDate, null));
      }

      _count = _pickerStateDetails._selectedRanges.length;
      _removeInterceptRanges(
          _pickerStateDetails._selectedRanges,
          _pickerStateDetails
              ._selectedRanges[_pickerStateDetails._selectedRanges.length - 1]);
      _lastRange = _pickerStateDetails
          ._selectedRanges[_pickerStateDetails._selectedRanges.length - 1];
      if (_count != _pickerStateDetails._selectedRanges.length &&
          (_lastRange.endDate == null ||
              isSameDate(_lastRange.endDate, _lastRange.startDate))) {
        _pickerStateDetails._selectedRanges.removeLast();
      }

      _monthView._updateSelection(_pickerStateDetails);
    }
  }

  void _removeInterceptRanges(List<PickerDateRange> _selectedRanges,
      PickerDateRange _selectedRangeValue) {
    if (_selectedRanges == null ||
        _selectedRanges.isEmpty ||
        _selectedRangeValue == null) {
      return;
    }

    DateTime _startDate = _selectedRangeValue.startDate;
    DateTime _endDate = _selectedRangeValue.endDate;
    if (_startDate != null &&
        _endDate != null &&
        _startDate.isAfter(_endDate)) {
      final DateTime temp = _startDate;
      _startDate = _endDate;
      _endDate = temp;
    }

    final List<int> _interceptIndex = <int>[];
    for (int i = 0; i < _selectedRanges.length; i++) {
      final PickerDateRange _range = _selectedRanges[i];
      //// The below condition validate the following scenarios
      //// Check the range as not null and range is not a new selected range,
      //// Check the range start date as equal with selected range start or end date
      //// Check the range end date as equal with selected range start or end date
      //// Check the selected start date placed in between range start or end date
      //// Check the selected end date placed in between range start or end date
      //// Check the selected range occupies the range.
      if (_range != null &&
          !_isRangeEquals(_range, _selectedRangeValue) &&
          ((_range.startDate != null &&
                  ((_startDate != null &&
                          isSameDate(_range.startDate, _startDate)) ||
                      (_endDate != null &&
                          isSameDate(_range.startDate, _endDate)))) ||
              (_range.endDate != null &&
                  ((_startDate != null &&
                          isSameDate(_range.endDate, _startDate)) ||
                      (_endDate != null &&
                          isSameDate(_range.endDate, _endDate)))) ||
              (_range.startDate != null &&
                  _range.endDate != null &&
                  ((_startDate != null &&
                          isDateWithInDateRange(
                              _range.startDate, _range.endDate, _startDate)) ||
                      (_endDate != null &&
                          isDateWithInDateRange(
                              _range.startDate, _range.endDate, _endDate)))) ||
              (_startDate != null &&
                  _endDate != null &&
                  ((_range.startDate != null &&
                          isDateWithInDateRange(
                              _startDate, _endDate, _range.startDate)) ||
                      (_range.endDate != null &&
                          isDateWithInDateRange(
                              _startDate, _endDate, _range.endDate)))) ||
              (_range.startDate != null &&
                  _range.endDate != null &&
                  _startDate != null &&
                  _endDate != null &&
                  ((_range.startDate.isAfter(_startDate) &&
                          _range.endDate.isBefore(_endDate)) ||
                      (_range.endDate.isAfter(_startDate) &&
                          _range.startDate.isBefore(_endDate)))))) {
        _interceptIndex.add(i);
      }
    }

    _interceptIndex.sort();
    for (int i = _interceptIndex.length - 1; i >= 0; i--) {
      _selectedRanges.removeAt(_interceptIndex[i]);
    }
  }
}
