part of datepicker;

class _PickerDecadeViewPainter extends _IPickerYearView {
  _PickerDecadeViewPainter(
      this.visibleDates,
      this.cellStyle,
      this.minDate,
      this.maxDate,
      this.enablePastDates,
      this.todayHighlightColor,
      this.selectionShape,
      this.isRtl,
      this.datePickerTheme,
      this._locale,
      this._mouseHoverPosition,
      this._selectionColor);

  final DateRangePickerYearCellStyle cellStyle;
  final List<DateTime> visibleDates;
  final bool isRtl;
  final Color todayHighlightColor;
  final DateTime minDate;
  final DateTime maxDate;
  final bool enablePastDates;
  final DateRangePickerSelectionShape selectionShape;
  final Offset _mouseHoverPosition;
  final Color _selectionColor;
  final Locale _locale;
  TextPainter _textPainter;
  Paint _todayHighlightPaint;
  final SfDateRangePickerThemeData datePickerTheme;

  @override
  void paint(Canvas canvas, Size size) {
    _drawYearCells(canvas, size, this);
  }

  @override
  bool _isToday(DateTime date, DateTime today) {
    if (date == null) {
      return false;
    }

    if (date.year == today.year) {
      return true;
    }

    return false;
  }

  @override
  bool _isBetweenMinMaxMonth(DateTime date) {
    if (date == null || minDate == null || maxDate == null) {
      return true;
    }

    final DateTime today = DateTime.now();
    if (date.year >= minDate.year &&
        date.year <= maxDate.year &&
        (enablePastDates || (!enablePastDates && date.year >= today.year))) {
      return true;
    }

    return false;
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    final _PickerDecadeViewPainter oldWidget = oldDelegate;
    return oldWidget.visibleDates != visibleDates ||
        oldWidget.todayHighlightColor != todayHighlightColor ||
        oldWidget.enablePastDates != enablePastDates ||
        oldWidget.cellStyle != cellStyle ||
        oldWidget.minDate != minDate ||
        oldWidget.maxDate != maxDate ||
        oldWidget.selectionShape != selectionShape ||
        oldWidget.isRtl != isRtl ||
        oldWidget.datePickerTheme != datePickerTheme ||
        oldWidget._locale != _locale ||
        (kIsWeb && oldWidget._mouseHoverPosition != _mouseHoverPosition);
  }

  @override
  String _getCellText(DateTime date) {
    return date.year.toString();
  }

  @override
  bool _isCurrentView(DateTime date) {
    final int currentYear = visibleDates[visibleDates.length ~/ 2].year;
    return currentYear ~/ 10 == date.year ~/ 10;
  }

  @override
  SemanticsBuilderCallback get semanticsBuilder {
    return (Size size) {
      return _getSemanticsBuilderForYearView(size, this);
    };
  }

  @override
  bool shouldRebuildSemantics(CustomPainter oldDelegate) {
    final _PickerDecadeViewPainter _oldWidget = oldDelegate;
    return _oldWidget.visibleDates != visibleDates;
  }
}
