part of datepicker;

void _drawYearCells(Canvas canvas, Size size, _IPickerYearView view) {
  canvas.clipRect(Rect.fromLTWH(0, 0, size.width, size.height));
  final double _cellWidth = size.width / _IPickerYearView._maxColumnCount;
  final double _cellHeight = size.height / _IPickerYearView._maxRowCount;

  final TextStyle _textStyle =
      view.cellStyle.textStyle ?? view.datePickerTheme.cellTextStyle;
  final TextStyle _leadingCellTextStyle =
      view.cellStyle.leadingDatesTextStyle ??
          view.datePickerTheme.leadingCellTextStyle;
  final TextStyle _disabledCellTextStyle =
      view.cellStyle.disabledDatesTextStyle ??
          view.datePickerTheme.disabledCellTextStyle;

  double _xPosition, _yPosition;
  _xPosition = 0;
  _yPosition = 0;
  final DateTime today = DateTime.now();
  view._textPainter ??= TextPainter(
      textAlign: TextAlign.start,
      textDirection: TextDirection.ltr,
      maxLines: 2,
      textWidthBasis: TextWidthBasis.longestLine);

  Decoration _yearDecoration;
  BoxPainter _boxPainter;
  const double padding = 1;
  for (int i = 0; i < view.visibleDates.length; i++) {
    int _currentIndex = i;
    if (view.isRtl) {
      final int rowIndex = i ~/ _IPickerYearView._maxColumnCount;
      _currentIndex = _getRtlIndex(_IPickerYearView._maxColumnCount,
              i % _IPickerYearView._maxColumnCount) +
          (rowIndex * _IPickerYearView._maxColumnCount);
    }

    final DateTime _date = view.visibleDates[_currentIndex];
    TextStyle _style = _textStyle;
    final bool _isCurrentDate = view._isToday(_date, today);
    _yearDecoration = view.cellStyle.cellDecoration;
    if (!view._isCurrentView(_date)) {
      _yearDecoration = view.cellStyle.leadingDatesDecoration;
      _style = _leadingCellTextStyle;
    }

    if (_isCurrentDate) {
      _yearDecoration = view.cellStyle.todayCellDecoration;
      _style = view.cellStyle.todayTextStyle ??
          view.datePickerTheme.todayCellTextStyle;
    }

    if (!view._isBetweenMinMaxMonth(_date)) {
      _yearDecoration = view.cellStyle.disabledDatesDecoration;
      _style = _disabledCellTextStyle;
    }

    final TextSpan _yearText = TextSpan(
      text: view._getCellText(_date),
      style: _style,
    );

    view._textPainter.text = _yearText;
    view._textPainter.layout(minWidth: 0, maxWidth: _cellWidth);

    double highlightPadding = 10;
    final double _centerXPosition = _cellWidth / 2;
    final double _centerYPosition = _cellHeight / 2;
    final double _textHalfWidth = view._textPainter.width / 2;
    final double _textHalfHeight = view._textPainter.height / 2;
    if (_yearDecoration != null) {
      _boxPainter = _yearDecoration.createBoxPainter();
      _boxPainter.paint(
          canvas,
          Offset(_xPosition + padding, _yPosition + padding),
          ImageConfiguration(
              size: Size(
                  _cellWidth - (2 * padding), _cellHeight - (2 * padding))));
    } else if (_isCurrentDate) {
      view._todayHighlightPaint ??= Paint();
      view._todayHighlightPaint.color =
          view.todayHighlightColor ?? view.datePickerTheme.todayHighlightColor;
      view._todayHighlightPaint.isAntiAlias = true;
      view._todayHighlightPaint.strokeWidth = 1.0;
      view._todayHighlightPaint.style = PaintingStyle.stroke;
      if (_centerXPosition - _textHalfWidth < highlightPadding / 2) {
        highlightPadding = (_centerXPosition - _textHalfWidth / 2) - 1;
      }

      if (_centerYPosition - _textHalfHeight < highlightPadding / 2) {
        highlightPadding = (_centerYPosition - _textHalfHeight / 2) - 1;
      }

      final Rect _rect = Rect.fromLTRB(
          _xPosition + _centerXPosition - highlightPadding - _textHalfWidth,
          _yPosition + _centerYPosition - highlightPadding - _textHalfHeight,
          _xPosition + _centerXPosition + highlightPadding + _textHalfWidth,
          _yPosition + _centerYPosition + highlightPadding + _textHalfHeight);
      double cornerRadius = _rect.height / 2;
      if (view.selectionShape == DateRangePickerSelectionShape.rectangle) {
        cornerRadius = 3;
      }

      canvas.drawRRect(
          RRect.fromRectAndRadius(_rect, Radius.circular(cornerRadius)),
          view._todayHighlightPaint);
    }

    double xOffset = _xPosition + ((_cellWidth - view._textPainter.width) / 2);
    xOffset = xOffset < 0 ? 0 : xOffset;
    double yOffset =
        _yPosition + ((_cellHeight - view._textPainter.height) / 2);
    yOffset = yOffset < 0 ? 0 : yOffset;

    if (view._mouseHoverPosition != null) {
      if (!view._isBetweenMinMaxMonth(_date)) {
        view._textPainter.paint(canvas, Offset(xOffset, yOffset));
        _xPosition += _cellWidth;
        if (_xPosition.round() >= size.width.round()) {
          _xPosition = 0;
          _yPosition += _cellHeight;
        }
        continue;
      }

      if (_xPosition <= view._mouseHoverPosition.dx &&
          _xPosition + _cellWidth >= view._mouseHoverPosition.dx &&
          _yPosition - 5 <= view._mouseHoverPosition.dy &&
          (_yPosition + _cellHeight) - 5 >= view._mouseHoverPosition.dy) {
        view._todayHighlightPaint = view._todayHighlightPaint ?? Paint();
        view._todayHighlightPaint.style = PaintingStyle.fill;
        view._todayHighlightPaint.strokeWidth = 2;
        view._todayHighlightPaint.color = view._selectionColor != null
            ? view._selectionColor.withOpacity(0.4)
            : view.datePickerTheme.selectionColor.withOpacity(0.4);
        if (_centerXPosition - _textHalfWidth < highlightPadding / 2) {
          highlightPadding = (_centerXPosition - _textHalfWidth / 2) - 1;
        }

        if (_centerYPosition - _textHalfHeight < highlightPadding / 2) {
          highlightPadding = (_centerYPosition - _textHalfHeight / 2) - 1;
        }

        final Rect _rect = Rect.fromLTRB(
            _xPosition + _centerXPosition - highlightPadding - _textHalfWidth,
            _yPosition + _centerYPosition - highlightPadding - _textHalfHeight,
            _xPosition + _centerXPosition + highlightPadding + _textHalfWidth,
            _yPosition + _centerYPosition + highlightPadding + _textHalfHeight);
        double cornerRadius = _rect.height / 2;
        if (view.selectionShape == DateRangePickerSelectionShape.rectangle) {
          cornerRadius = 3;
        }

        canvas.drawRRect(
            RRect.fromRectAndRadius(_rect, Radius.circular(cornerRadius)),
            view._todayHighlightPaint);
      }
    }

    view._textPainter.paint(canvas, Offset(xOffset, yOffset));
    _xPosition += _cellWidth;
    if (_xPosition.round() >= size.width.round()) {
      _xPosition = 0;
      _yPosition += _cellHeight;
    }
  }
}

List<CustomPainterSemantics> _getSemanticsBuilderForYearView(
    Size size, _IPickerYearView view) {
  final List<CustomPainterSemantics> _semanticsBuilder =
      <CustomPainterSemantics>[];
  double _left, _top, _cellWidth, _cellHeight;
  Map<String, double> _leftAndTopValue;
  _top = 0;
  _cellWidth = size.width / 3;
  _cellHeight = size.height / 4;
  _left = view.isRtl ? size.width - _cellWidth : 0;
  for (int i = 0; i < view.visibleDates.length; i++) {
    if (!view._isBetweenMinMaxMonth(view.visibleDates[i])) {
      _semanticsBuilder.add(CustomPainterSemantics(
        rect: Rect.fromLTWH(_left, _top, _cellWidth, _cellHeight),
        properties: SemanticsProperties(
          label: _getCellText(view.visibleDates[i], view) + 'Disabled cell',
          textDirection: TextDirection.ltr,
        ),
      ));

      _leftAndTopValue = _getTopAndLeftValues(
          view.isRtl, _left, _top, _cellWidth, _cellHeight, size.width);
      _left = _leftAndTopValue['left'];
      _top = _leftAndTopValue['top'];
      continue;
    }
    _semanticsBuilder.add(CustomPainterSemantics(
      rect: Rect.fromLTWH(_left, _top, _cellWidth, _cellHeight),
      properties: SemanticsProperties(
        label: _getCellText(view.visibleDates[i], view),
        textDirection: TextDirection.ltr,
      ),
    ));
    _leftAndTopValue = _getTopAndLeftValues(
        view.isRtl, _left, _top, _cellWidth, _cellHeight, size.width);
    _left = _leftAndTopValue['left'];
    _top = _leftAndTopValue['top'];
  }

  return _semanticsBuilder;
}

String _getCellText(DateTime date, _IPickerYearView view) {
  if (view.runtimeType.toString() == '_PickerYearViewPainter') {
    return DateFormat('MMMM yyyy').format(date).toString();
  } else if (view.runtimeType.toString() == '_PickerDecadeViewPainter') {
    return date.year.toString();
  } else {
    return date.year.toString() + ' to ' + (date.year + 9).toString();
  }
}
